# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       MHL
@license:      GNU General Public License 2.0
@contact:      michael.ligh@mnin.org

This file provides support for Windows 2003 SP1/2.
"""

syscalls = [
	[
	'NtAcceptConnectPort', # 0x0
	'NtAccessCheck', # 0x1
	'NtAccessCheckAndAuditAlarm', # 0x2
	'NtAccessCheckByType', # 0x3
	'NtAccessCheckByTypeAndAuditAlarm', # 0x4
	'NtAccessCheckByTypeResultList', # 0x5
	'NtAccessCheckByTypeResultListAndAuditAlarm', # 0x6
	'NtAccessCheckByTypeResultListAndAuditAlarmByHandle', # 0x7
	'NtAddAtom', # 0x8
	'NtAddBootEntry', # 0x9
	'NtAddDriverEntry', # 0xa
	'NtAdjustGroupsToken', # 0xb
	'NtAdjustPrivilegesToken', # 0xc
	'NtAlertResumeThread', # 0xd
	'NtAlertThread', # 0xe
	'NtAllocateLocallyUniqueId', # 0xf
	'NtAllocateUserPhysicalPages', # 0x10
	'NtAllocateUuids', # 0x11
	'NtAllocateVirtualMemory', # 0x12
	'NtApphelpCacheControl', # 0x13
	'NtAreMappedFilesTheSame', # 0x14
	'NtAssignProcessToJobObject', # 0x15
	'NtCallbackReturn', # 0x16
	'NtCancelDeviceWakeupRequest', # 0x17
	'NtCancelIoFile', # 0x18
	'NtCancelTimer', # 0x19
	'NtClearEvent', # 0x1a
	'NtClose', # 0x1b
	'NtCloseObjectAuditAlarm', # 0x1c
	'NtCompactKeys', # 0x1d
	'NtCompareTokens', # 0x1e
	'NtCompleteConnectPort', # 0x1f
	'NtCompressKey', # 0x20
	'NtConnectPort', # 0x21
	'NtContinue', # 0x22
	'NtCreateDebugObject', # 0x23
	'NtCreateDirectoryObject', # 0x24
	'NtCreateEvent', # 0x25
	'NtCreateEventPair', # 0x26
	'NtCreateFile', # 0x27
	'NtCreateIoCompletion', # 0x28
	'NtCreateJobObject', # 0x29
	'NtCreateJobSet', # 0x2a
	'NtCreateKey', # 0x2b
	'NtCreateMailslotFile', # 0x2c
	'NtCreateMutant', # 0x2d
	'NtCreateNamedPipeFile', # 0x2e
	'NtCreatePagingFile', # 0x2f
	'NtCreatePort', # 0x30
	'NtCreateProcess', # 0x31
	'NtCreateProcessEx', # 0x32
	'NtCreateProfile', # 0x33
	'NtCreateSection', # 0x34
	'NtCreateSemaphore', # 0x35
	'NtCreateSymbolicLinkObject', # 0x36
	'NtCreateThread', # 0x37
	'NtCreateTimer', # 0x38
	'NtCreateToken', # 0x39
	'NtCreateWaitablePort', # 0x3a
	'NtDebugActiveProcess', # 0x3b
	'NtDebugContinue', # 0x3c
	'NtDelayExecution', # 0x3d
	'NtDeleteAtom', # 0x3e
	'NtDeleteBootEntry', # 0x3f
	'NtDeleteDriverEntry', # 0x40
	'NtDeleteFile', # 0x41
	'NtDeleteKey', # 0x42
	'NtDeleteObjectAuditAlarm', # 0x43
	'NtDeleteValueKey', # 0x44
	'NtDeviceIoControlFile', # 0x45
	'NtDisplayString', # 0x46
	'NtDuplicateObject', # 0x47
	'NtDuplicateToken', # 0x48
	'NtEnumerateBootEntries', # 0x49
	'NtEnumerateDriverEntries', # 0x4a
	'NtEnumerateKey', # 0x4b
	'NtEnumerateSystemEnvironmentValuesEx', # 0x4c
	'NtEnumerateValueKey', # 0x4d
	'NtExtendSection', # 0x4e
	'NtFilterToken', # 0x4f
	'NtFindAtom', # 0x50
	'NtFlushBuffersFile', # 0x51
	'NtFlushInstructionCache', # 0x52
	'NtFlushKey', # 0x53
	'NtFlushVirtualMemory', # 0x54
	'NtFlushWriteBuffer', # 0x55
	'NtFreeUserPhysicalPages', # 0x56
	'NtFreeVirtualMemory', # 0x57
	'NtFsControlFile', # 0x58
	'NtGetContextThread', # 0x59
	'NtGetDevicePowerState', # 0x5a
	'NtGetPlugPlayEvent', # 0x5b
	'NtGetWriteWatch', # 0x5c
	'NtImpersonateAnonymousToken', # 0x5d
	'NtImpersonateClientOfPort', # 0x5e
	'NtImpersonateThread', # 0x5f
	'NtInitializeRegistry', # 0x60
	'NtInitiatePowerAction', # 0x61
	'NtIsProcessInJob', # 0x62
	'NtIsSystemResumeAutomatic', # 0x63
	'NtListenPort', # 0x64
	'NtLoadDriver', # 0x65
	'NtLoadKey', # 0x66
	'NtLoadKey2', # 0x67
	'NtLoadKeyEx', # 0x68
	'NtLockFile', # 0x69
	'NtLockProductActivationKeys', # 0x6a
	'NtLockRegistryKey', # 0x6b
	'NtLockVirtualMemory', # 0x6c
	'NtMakePermanentObject', # 0x6d
	'NtMakeTemporaryObject', # 0x6e
	'NtMapUserPhysicalPages', # 0x6f
	'NtMapUserPhysicalPagesScatter', # 0x70
	'NtMapViewOfSection', # 0x71
	'NtModifyBootEntry', # 0x72
	'NtModifyDriverEntry', # 0x73
	'NtNotifyChangeDirectoryFile', # 0x74
	'NtNotifyChangeKey', # 0x75
	'NtNotifyChangeMultipleKeys', # 0x76
	'NtOpenDirectoryObject', # 0x77
	'NtOpenEvent', # 0x78
	'NtOpenEventPair', # 0x79
	'NtOpenFile', # 0x7a
	'NtOpenIoCompletion', # 0x7b
	'NtOpenJobObject', # 0x7c
	'NtOpenKey', # 0x7d
	'NtOpenMutant', # 0x7e
	'NtOpenObjectAuditAlarm', # 0x7f
	'NtOpenProcess', # 0x80
	'NtOpenProcessToken', # 0x81
	'NtOpenProcessTokenEx', # 0x82
	'NtOpenSection', # 0x83
	'NtOpenSemaphore', # 0x84
	'NtOpenSymbolicLinkObject', # 0x85
	'NtOpenThread', # 0x86
	'NtOpenThreadToken', # 0x87
	'NtOpenThreadTokenEx', # 0x88
	'NtOpenTimer', # 0x89
	'NtPlugPlayControl', # 0x8a
	'NtPowerInformation', # 0x8b
	'NtPrivilegeCheck', # 0x8c
	'NtPrivilegeObjectAuditAlarm', # 0x8d
	'NtPrivilegedServiceAuditAlarm', # 0x8e
	'NtProtectVirtualMemory', # 0x8f
	'NtPulseEvent', # 0x90
	'NtQueryAttributesFile', # 0x91
	'NtQueryBootEntryOrder', # 0x92
	'NtQueryBootOptions', # 0x93
	'NtQueryDebugFilterState', # 0x94
	'NtQueryDefaultLocale', # 0x95
	'NtQueryDefaultUILanguage', # 0x96
	'NtQueryDirectoryFile', # 0x97
	'NtQueryDirectoryObject', # 0x98
	'NtQueryDriverEntryOrder', # 0x99
	'NtQueryEaFile', # 0x9a
	'NtQueryEvent', # 0x9b
	'NtQueryFullAttributesFile', # 0x9c
	'NtQueryInformationAtom', # 0x9d
	'NtQueryInformationFile', # 0x9e
	'NtQueryInformationJobObject', # 0x9f
	'NtQueryInformationPort', # 0xa0
	'NtQueryInformationProcess', # 0xa1
	'NtQueryInformationThread', # 0xa2
	'NtQueryInformationToken', # 0xa3
	'NtQueryInstallUILanguage', # 0xa4
	'NtQueryIntervalProfile', # 0xa5
	'NtQueryIoCompletion', # 0xa6
	'NtQueryKey', # 0xa7
	'NtQueryMultipleValueKey', # 0xa8
	'NtQueryMutant', # 0xa9
	'NtQueryObject', # 0xaa
	'NtQueryOpenSubKeys', # 0xab
	'NtQueryOpenSubKeysEx', # 0xac
	'NtQueryPerformanceCounter', # 0xad
	'NtQueryQuotaInformationFile', # 0xae
	'NtQuerySection', # 0xaf
	'NtQuerySecurityObject', # 0xb0
	'NtQuerySemaphore', # 0xb1
	'NtQuerySymbolicLinkObject', # 0xb2
	'NtQuerySystemEnvironmentValue', # 0xb3
	'NtQuerySystemEnvironmentValueEx', # 0xb4
	'NtQuerySystemInformation', # 0xb5
	'NtQuerySystemTime', # 0xb6
	'NtQueryTimer', # 0xb7
	'NtQueryTimerResolution', # 0xb8
	'NtQueryValueKey', # 0xb9
	'NtQueryVirtualMemory', # 0xba
	'NtQueryVolumeInformationFile', # 0xbb
	'NtQueueApcThread', # 0xbc
	'NtRaiseException', # 0xbd
	'NtRaiseHardError', # 0xbe
	'NtReadFile', # 0xbf
	'NtReadFileScatter', # 0xc0
	'NtReadRequestData', # 0xc1
	'NtReadVirtualMemory', # 0xc2
	'NtRegisterThreadTerminatePort', # 0xc3
	'NtReleaseMutant', # 0xc4
	'NtReleaseSemaphore', # 0xc5
	'NtRemoveIoCompletion', # 0xc6
	'NtRemoveProcessDebug', # 0xc7
	'NtRenameKey', # 0xc8
	'NtReplaceKey', # 0xc9
	'NtReplyPort', # 0xca
	'NtReplyWaitReceivePort', # 0xcb
	'NtReplyWaitReceivePortEx', # 0xcc
	'NtReplyWaitReplyPort', # 0xcd
	'NtRequestDeviceWakeup', # 0xce
	'NtRequestPort', # 0xcf
	'NtRequestWaitReplyPort', # 0xd0
	'NtRequestWakeupLatency', # 0xd1
	'NtResetEvent', # 0xd2
	'NtResetWriteWatch', # 0xd3
	'NtRestoreKey', # 0xd4
	'NtResumeProcess', # 0xd5
	'NtResumeThread', # 0xd6
	'NtSaveKey', # 0xd7
	'NtSaveKeyEx', # 0xd8
	'NtSaveMergedKeys', # 0xd9
	'NtSecureConnectPort', # 0xda
	'NtSetBootEntryOrder', # 0xdb
	'NtSetBootOptions', # 0xdc
	'NtSetContextThread', # 0xdd
	'NtSetDebugFilterState', # 0xde
	'NtSetDefaultHardErrorPort', # 0xdf
	'NtSetDefaultLocale', # 0xe0
	'NtSetDefaultUILanguage', # 0xe1
	'NtSetDriverEntryOrder', # 0xe2
	'NtSetEaFile', # 0xe3
	'NtSetEvent', # 0xe4
	'NtSetEventBoostPriority', # 0xe5
	'NtSetHighEventPair', # 0xe6
	'NtSetHighWaitLowEventPair', # 0xe7
	'NtSetInformationDebugObject', # 0xe8
	'NtSetInformationFile', # 0xe9
	'NtSetInformationJobObject', # 0xea
	'NtSetInformationKey', # 0xeb
	'NtSetInformationObject', # 0xec
	'NtSetInformationProcess', # 0xed
	'NtSetInformationThread', # 0xee
	'NtSetInformationToken', # 0xef
	'NtSetIntervalProfile', # 0xf0
	'NtSetIoCompletion', # 0xf1
	'NtSetLdtEntries', # 0xf2
	'NtSetLowEventPair', # 0xf3
	'NtSetLowWaitHighEventPair', # 0xf4
	'NtSetQuotaInformationFile', # 0xf5
	'NtSetSecurityObject', # 0xf6
	'NtSetSystemEnvironmentValue', # 0xf7
	'NtSetSystemEnvironmentValueEx', # 0xf8
	'NtSetSystemInformation', # 0xf9
	'NtSetSystemPowerState', # 0xfa
	'NtSetSystemTime', # 0xfb
	'NtSetThreadExecutionState', # 0xfc
	'NtSetTimer', # 0xfd
	'NtSetTimerResolution', # 0xfe
	'NtSetUuidSeed', # 0xff
	'NtSetValueKey', # 0x100
	'NtSetVolumeInformationFile', # 0x101
	'NtShutdownSystem', # 0x102
	'NtSignalAndWaitForSingleObject', # 0x103
	'NtStartProfile', # 0x104
	'NtStopProfile', # 0x105
	'NtSuspendProcess', # 0x106
	'NtSuspendThread', # 0x107
	'NtSystemDebugControl', # 0x108
	'NtTerminateJobObject', # 0x109
	'NtTerminateProcess', # 0x10a
	'NtTerminateThread', # 0x10b
	'NtTestAlert', # 0x10c
	'NtTraceEvent', # 0x10d
	'NtTranslateFilePath', # 0x10e
	'NtUnloadDriver', # 0x10f
	'NtUnloadKey', # 0x110
	'NtUnloadKey2', # 0x111
	'NtUnloadKeyEx', # 0x112
	'NtUnlockFile', # 0x113
	'NtUnlockVirtualMemory', # 0x114
	'NtUnmapViewOfSection', # 0x115
	'NtVdmControl', # 0x116
	'NtWaitForDebugEvent', # 0x117
	'NtWaitForMultipleObjects', # 0x118
	'NtWaitForSingleObject', # 0x119
	'NtWaitHighEventPair', # 0x11a
	'NtWaitLowEventPair', # 0x11b
	'NtWriteFile', # 0x11c
	'NtWriteFileGather', # 0x11d
	'NtWriteRequestData', # 0x11e
	'NtWriteVirtualMemory', # 0x11f
	'NtYieldExecution', # 0x120
	'NtCreateKeyedEvent', # 0x121
	'NtOpenKeyedEvent', # 0x122
	'NtReleaseKeyedEvent', # 0x123
	'NtWaitForKeyedEvent', # 0x124
	'NtQueryPortInformationProcess', # 0x125
	'NtGetCurrentProcessorNumber', # 0x126
	'NtWaitForMultipleObjects32', # 0x127
	],
	[
	'NtGdiAbortDoc', # 0x0
	'NtGdiAbortPath', # 0x1
	'NtGdiAddFontResourceW', # 0x2
	'NtGdiAddRemoteFontToDC', # 0x3
	'NtGdiAddFontMemResourceEx', # 0x4
	'NtGdiRemoveMergeFont', # 0x5
	'NtGdiAddRemoteMMInstanceToDC', # 0x6
	'NtGdiAlphaBlend', # 0x7
	'NtGdiAngleArc', # 0x8
	'NtGdiAnyLinkedFonts', # 0x9
	'NtGdiFontIsLinked', # 0xa
	'NtGdiArcInternal', # 0xb
	'NtGdiBeginPath', # 0xc
	'NtGdiBitBlt', # 0xd
	'NtGdiCancelDC', # 0xe
	'NtGdiCheckBitmapBits', # 0xf
	'NtGdiCloseFigure', # 0x10
	'NtGdiClearBitmapAttributes', # 0x11
	'NtGdiClearBrushAttributes', # 0x12
	'NtGdiColorCorrectPalette', # 0x13
	'NtGdiCombineRgn', # 0x14
	'NtGdiCombineTransform', # 0x15
	'NtGdiComputeXformCoefficients', # 0x16
	'NtGdiConsoleTextOut', # 0x17
	'NtGdiConvertMetafileRect', # 0x18
	'NtGdiCreateBitmap', # 0x19
	'NtGdiCreateClientObj', # 0x1a
	'NtGdiCreateColorSpace', # 0x1b
	'NtGdiCreateColorTransform', # 0x1c
	'NtGdiCreateCompatibleBitmap', # 0x1d
	'NtGdiCreateCompatibleDC', # 0x1e
	'NtGdiCreateDIBBrush', # 0x1f
	'NtGdiCreateDIBitmapInternal', # 0x20
	'NtGdiCreateDIBSection', # 0x21
	'NtGdiCreateEllipticRgn', # 0x22
	'NtGdiCreateHalftonePalette', # 0x23
	'NtGdiCreateHatchBrushInternal', # 0x24
	'NtGdiCreateMetafileDC', # 0x25
	'NtGdiCreatePaletteInternal', # 0x26
	'NtGdiCreatePatternBrushInternal', # 0x27
	'NtGdiCreatePen', # 0x28
	'NtGdiCreateRectRgn', # 0x29
	'NtGdiCreateRoundRectRgn', # 0x2a
	'NtGdiCreateServerMetaFile', # 0x2b
	'NtGdiCreateSolidBrush', # 0x2c
	'NtGdiD3dContextCreate', # 0x2d
	'NtGdiD3dContextDestroy', # 0x2e
	'NtGdiD3dContextDestroyAll', # 0x2f
	'NtGdiD3dValidateTextureStageState', # 0x30
	'NtGdiD3dDrawPrimitives2', # 0x31
	'NtGdiDdGetDriverState', # 0x32
	'NtGdiDdAddAttachedSurface', # 0x33
	'NtGdiDdAlphaBlt', # 0x34
	'NtGdiDdAttachSurface', # 0x35
	'NtGdiDdBeginMoCompFrame', # 0x36
	'NtGdiDdBlt', # 0x37
	'NtGdiDdCanCreateSurface', # 0x38
	'NtGdiDdCanCreateD3DBuffer', # 0x39
	'NtGdiDdColorControl', # 0x3a
	'NtGdiDdCreateDirectDrawObject', # 0x3b
	'NtGdiDdCreateSurface', # 0x3c
	'NtGdiDdCreateD3DBuffer', # 0x3d
	'NtGdiDdCreateMoComp', # 0x3e
	'NtGdiDdCreateSurfaceObject', # 0x3f
	'NtGdiDdDeleteDirectDrawObject', # 0x40
	'NtGdiDdDeleteSurfaceObject', # 0x41
	'NtGdiDdDestroyMoComp', # 0x42
	'NtGdiDdDestroySurface', # 0x43
	'NtGdiDdDestroyD3DBuffer', # 0x44
	'NtGdiDdEndMoCompFrame', # 0x45
	'NtGdiDdFlip', # 0x46
	'NtGdiDdFlipToGDISurface', # 0x47
	'NtGdiDdGetAvailDriverMemory', # 0x48
	'NtGdiDdGetBltStatus', # 0x49
	'NtGdiDdGetDC', # 0x4a
	'NtGdiDdGetDriverInfo', # 0x4b
	'NtGdiDdGetDxHandle', # 0x4c
	'NtGdiDdGetFlipStatus', # 0x4d
	'NtGdiDdGetInternalMoCompInfo', # 0x4e
	'NtGdiDdGetMoCompBuffInfo', # 0x4f
	'NtGdiDdGetMoCompGuids', # 0x50
	'NtGdiDdGetMoCompFormats', # 0x51
	'NtGdiDdGetScanLine', # 0x52
	'NtGdiDdLock', # 0x53
	'NtGdiDdLockD3D', # 0x54
	'NtGdiDdQueryDirectDrawObject', # 0x55
	'NtGdiDdQueryMoCompStatus', # 0x56
	'NtGdiDdReenableDirectDrawObject', # 0x57
	'NtGdiDdReleaseDC', # 0x58
	'NtGdiDdRenderMoComp', # 0x59
	'NtGdiDdResetVisrgn', # 0x5a
	'NtGdiDdSetColorKey', # 0x5b
	'NtGdiDdSetExclusiveMode', # 0x5c
	'NtGdiDdSetGammaRamp', # 0x5d
	'NtGdiDdCreateSurfaceEx', # 0x5e
	'NtGdiDdSetOverlayPosition', # 0x5f
	'NtGdiDdUnattachSurface', # 0x60
	'NtGdiDdUnlock', # 0x61
	'NtGdiDdUnlockD3D', # 0x62
	'NtGdiDdUpdateOverlay', # 0x63
	'NtGdiDdWaitForVerticalBlank', # 0x64
	'NtGdiDvpCanCreateVideoPort', # 0x65
	'NtGdiDvpColorControl', # 0x66
	'NtGdiDvpCreateVideoPort', # 0x67
	'NtGdiDvpDestroyVideoPort', # 0x68
	'NtGdiDvpFlipVideoPort', # 0x69
	'NtGdiDvpGetVideoPortBandwidth', # 0x6a
	'NtGdiDvpGetVideoPortField', # 0x6b
	'NtGdiDvpGetVideoPortFlipStatus', # 0x6c
	'NtGdiDvpGetVideoPortInputFormats', # 0x6d
	'NtGdiDvpGetVideoPortLine', # 0x6e
	'NtGdiDvpGetVideoPortOutputFormats', # 0x6f
	'NtGdiDvpGetVideoPortConnectInfo', # 0x70
	'NtGdiDvpGetVideoSignalStatus', # 0x71
	'NtGdiDvpUpdateVideoPort', # 0x72
	'NtGdiDvpWaitForVideoPortSync', # 0x73
	'NtGdiDvpAcquireNotification', # 0x74
	'NtGdiDvpReleaseNotification', # 0x75
	'NtGdiDxgGenericThunk', # 0x76
	'NtGdiDeleteClientObj', # 0x77
	'NtGdiDeleteColorSpace', # 0x78
	'NtGdiDeleteColorTransform', # 0x79
	'NtGdiDeleteObjectApp', # 0x7a
	'NtGdiDescribePixelFormat', # 0x7b
	'NtGdiGetPerBandInfo', # 0x7c
	'NtGdiDoBanding', # 0x7d
	'NtGdiDoPalette', # 0x7e
	'NtGdiDrawEscape', # 0x7f
	'NtGdiEllipse', # 0x80
	'NtGdiEnableEudc', # 0x81
	'NtGdiEndDoc', # 0x82
	'NtGdiEndPage', # 0x83
	'NtGdiEndPath', # 0x84
	'NtGdiEnumFontChunk', # 0x85
	'NtGdiEnumFontClose', # 0x86
	'NtGdiEnumFontOpen', # 0x87
	'NtGdiEnumObjects', # 0x88
	'NtGdiEqualRgn', # 0x89
	'NtGdiEudcLoadUnloadLink', # 0x8a
	'NtGdiExcludeClipRect', # 0x8b
	'NtGdiExtCreatePen', # 0x8c
	'NtGdiExtCreateRegion', # 0x8d
	'NtGdiExtEscape', # 0x8e
	'NtGdiExtFloodFill', # 0x8f
	'NtGdiExtGetObjectW', # 0x90
	'NtGdiExtSelectClipRgn', # 0x91
	'NtGdiExtTextOutW', # 0x92
	'NtGdiFillPath', # 0x93
	'NtGdiFillRgn', # 0x94
	'NtGdiFlattenPath', # 0x95
	'NtGdiFlush', # 0x96
	'NtGdiForceUFIMapping', # 0x97
	'NtGdiFrameRgn', # 0x98
	'NtGdiFullscreenControl', # 0x99
	'NtGdiGetAndSetDCDword', # 0x9a
	'NtGdiGetAppClipBox', # 0x9b
	'NtGdiGetBitmapBits', # 0x9c
	'NtGdiGetBitmapDimension', # 0x9d
	'NtGdiGetBoundsRect', # 0x9e
	'NtGdiGetCharABCWidthsW', # 0x9f
	'NtGdiGetCharacterPlacementW', # 0xa0
	'NtGdiGetCharSet', # 0xa1
	'NtGdiGetCharWidthW', # 0xa2
	'NtGdiGetCharWidthInfo', # 0xa3
	'NtGdiGetColorAdjustment', # 0xa4
	'NtGdiGetColorSpaceforBitmap', # 0xa5
	'NtGdiGetDCDword', # 0xa6
	'NtGdiGetDCforBitmap', # 0xa7
	'NtGdiGetDCObject', # 0xa8
	'NtGdiGetDCPoint', # 0xa9
	'NtGdiGetDeviceCaps', # 0xaa
	'NtGdiGetDeviceGammaRamp', # 0xab
	'NtGdiGetDeviceCapsAll', # 0xac
	'NtGdiGetDIBitsInternal', # 0xad
	'NtGdiGetETM', # 0xae
	'NtGdiGetEudcTimeStampEx', # 0xaf
	'NtGdiGetFontData', # 0xb0
	'NtGdiGetFontResourceInfoInternalW', # 0xb1
	'NtGdiGetGlyphIndicesW', # 0xb2
	'NtGdiGetGlyphIndicesWInternal', # 0xb3
	'NtGdiGetGlyphOutline', # 0xb4
	'NtGdiGetKerningPairs', # 0xb5
	'NtGdiGetLinkedUFIs', # 0xb6
	'NtGdiGetMiterLimit', # 0xb7
	'NtGdiGetMonitorID', # 0xb8
	'NtGdiGetNearestColor', # 0xb9
	'NtGdiGetNearestPaletteIndex', # 0xba
	'NtGdiGetObjectBitmapHandle', # 0xbb
	'NtGdiGetOutlineTextMetricsInternalW', # 0xbc
	'NtGdiGetPath', # 0xbd
	'NtGdiGetPixel', # 0xbe
	'NtGdiGetRandomRgn', # 0xbf
	'NtGdiGetRasterizerCaps', # 0xc0
	'NtGdiGetRealizationInfo', # 0xc1
	'NtGdiGetRegionData', # 0xc2
	'NtGdiGetRgnBox', # 0xc3
	'NtGdiGetServerMetaFileBits', # 0xc4
	'NtGdiGetSpoolMessage', # 0xc5
	'NtGdiGetStats', # 0xc6
	'NtGdiGetStockObject', # 0xc7
	'NtGdiGetStringBitmapW', # 0xc8
	'NtGdiGetSystemPaletteUse', # 0xc9
	'NtGdiGetTextCharsetInfo', # 0xca
	'NtGdiGetTextExtent', # 0xcb
	'NtGdiGetTextExtentExW', # 0xcc
	'NtGdiGetTextFaceW', # 0xcd
	'NtGdiGetTextMetricsW', # 0xce
	'NtGdiGetTransform', # 0xcf
	'NtGdiGetUFI', # 0xd0
	'NtGdiGetEmbUFI', # 0xd1
	'NtGdiGetUFIPathname', # 0xd2
	'NtGdiGetEmbedFonts', # 0xd3
	'NtGdiChangeGhostFont', # 0xd4
	'NtGdiAddEmbFontToDC', # 0xd5
	'NtGdiGetFontUnicodeRanges', # 0xd6
	'NtGdiGetWidthTable', # 0xd7
	'NtGdiGradientFill', # 0xd8
	'NtGdiHfontCreate', # 0xd9
	'NtGdiIcmBrushInfo', # 0xda
	'NtGdiInit', # 0xdb
	'NtGdiInitSpool', # 0xdc
	'NtGdiIntersectClipRect', # 0xdd
	'NtGdiInvertRgn', # 0xde
	'NtGdiLineTo', # 0xdf
	'NtGdiMakeFontDir', # 0xe0
	'NtGdiMakeInfoDC', # 0xe1
	'NtGdiMaskBlt', # 0xe2
	'NtGdiModifyWorldTransform', # 0xe3
	'NtGdiMonoBitmap', # 0xe4
	'NtGdiMoveTo', # 0xe5
	'NtGdiOffsetClipRgn', # 0xe6
	'NtGdiOffsetRgn', # 0xe7
	'NtGdiOpenDCW', # 0xe8
	'NtGdiPatBlt', # 0xe9
	'NtGdiPolyPatBlt', # 0xea
	'NtGdiPathToRegion', # 0xeb
	'NtGdiPlgBlt', # 0xec
	'NtGdiPolyDraw', # 0xed
	'NtGdiPolyPolyDraw', # 0xee
	'NtGdiPolyTextOutW', # 0xef
	'NtGdiPtInRegion', # 0xf0
	'NtGdiPtVisible', # 0xf1
	'NtGdiQueryFonts', # 0xf2
	'NtGdiQueryFontAssocInfo', # 0xf3
	'NtGdiRectangle', # 0xf4
	'NtGdiRectInRegion', # 0xf5
	'NtGdiRectVisible', # 0xf6
	'NtGdiRemoveFontResourceW', # 0xf7
	'NtGdiRemoveFontMemResourceEx', # 0xf8
	'NtGdiResetDC', # 0xf9
	'NtGdiResizePalette', # 0xfa
	'NtGdiRestoreDC', # 0xfb
	'NtGdiRoundRect', # 0xfc
	'NtGdiSaveDC', # 0xfd
	'NtGdiScaleViewportExtEx', # 0xfe
	'NtGdiScaleWindowExtEx', # 0xff
	'NtGdiSelectBitmap', # 0x100
	'NtGdiSelectBrush', # 0x101
	'NtGdiSelectClipPath', # 0x102
	'NtGdiSelectFont', # 0x103
	'NtGdiSelectPen', # 0x104
	'NtGdiSetBitmapAttributes', # 0x105
	'NtGdiSetBitmapBits', # 0x106
	'NtGdiSetBitmapDimension', # 0x107
	'NtGdiSetBoundsRect', # 0x108
	'NtGdiSetBrushAttributes', # 0x109
	'NtGdiSetBrushOrg', # 0x10a
	'NtGdiSetColorAdjustment', # 0x10b
	'NtGdiSetColorSpace', # 0x10c
	'NtGdiSetDeviceGammaRamp', # 0x10d
	'NtGdiSetDIBitsToDeviceInternal', # 0x10e
	'NtGdiSetFontEnumeration', # 0x10f
	'NtGdiSetFontXform', # 0x110
	'NtGdiSetIcmMode', # 0x111
	'NtGdiSetLinkedUFIs', # 0x112
	'NtGdiSetMagicColors', # 0x113
	'NtGdiSetMetaRgn', # 0x114
	'NtGdiSetMiterLimit', # 0x115
	'NtGdiGetDeviceWidth', # 0x116
	'NtGdiMirrorWindowOrg', # 0x117
	'NtGdiSetLayout', # 0x118
	'NtGdiSetPixel', # 0x119
	'NtGdiSetPixelFormat', # 0x11a
	'NtGdiSetRectRgn', # 0x11b
	'NtGdiSetSystemPaletteUse', # 0x11c
	'NtGdiSetTextJustification', # 0x11d
	'NtGdiSetupPublicCFONT', # 0x11e
	'NtGdiSetVirtualResolution', # 0x11f
	'NtGdiSetSizeDevice', # 0x120
	'NtGdiStartDoc', # 0x121
	'NtGdiStartPage', # 0x122
	'NtGdiStretchBlt', # 0x123
	'NtGdiStretchDIBitsInternal', # 0x124
	'NtGdiStrokeAndFillPath', # 0x125
	'NtGdiStrokePath', # 0x126
	'NtGdiSwapBuffers', # 0x127
	'NtGdiTransformPoints', # 0x128
	'NtGdiTransparentBlt', # 0x129
	'NtGdiUnloadPrinterDriver', # 0x12a
	'NtGdiUnmapMemFont', # 0x12b
	'NtGdiUnrealizeObject', # 0x12c
	'NtGdiUpdateColors', # 0x12d
	'NtGdiWidenPath', # 0x12e
	'NtUserActivateKeyboardLayout', # 0x12f
	'NtUserAlterWindowStyle', # 0x130
	'NtUserAssociateInputContext', # 0x131
	'NtUserAttachThreadInput', # 0x132
	'NtUserBeginPaint', # 0x133
	'NtUserBitBltSysBmp', # 0x134
	'NtUserBlockInput', # 0x135
	'NtUserBuildHimcList', # 0x136
	'NtUserBuildHwndList', # 0x137
	'NtUserBuildNameList', # 0x138
	'NtUserBuildPropList', # 0x139
	'NtUserCallHwnd', # 0x13a
	'NtUserCallHwndLock', # 0x13b
	'NtUserCallHwndOpt', # 0x13c
	'NtUserCallHwndParam', # 0x13d
	'NtUserCallHwndParamLock', # 0x13e
	'NtUserCallMsgFilter', # 0x13f
	'NtUserCallNextHookEx', # 0x140
	'NtUserCallNoParam', # 0x141
	'NtUserCallOneParam', # 0x142
	'NtUserCallTwoParam', # 0x143
	'NtUserChangeClipboardChain', # 0x144
	'NtUserChangeDisplaySettings', # 0x145
	'NtUserCheckImeHotKey', # 0x146
	'NtUserCheckMenuItem', # 0x147
	'NtUserChildWindowFromPointEx', # 0x148
	'NtUserClipCursor', # 0x149
	'NtUserCloseClipboard', # 0x14a
	'NtUserCloseDesktop', # 0x14b
	'NtUserCloseWindowStation', # 0x14c
	'NtUserConsoleControl', # 0x14d
	'NtUserConvertMemHandle', # 0x14e
	'NtUserCopyAcceleratorTable', # 0x14f
	'NtUserCountClipboardFormats', # 0x150
	'NtUserCreateAcceleratorTable', # 0x151
	'NtUserCreateCaret', # 0x152
	'NtUserCreateDesktop', # 0x153
	'NtUserCreateInputContext', # 0x154
	'NtUserCreateLocalMemHandle', # 0x155
	'NtUserCreateWindowEx', # 0x156
	'NtUserCreateWindowStation', # 0x157
	'NtUserDdeGetQualityOfService', # 0x158
	'NtUserDdeInitialize', # 0x159
	'NtUserDdeSetQualityOfService', # 0x15a
	'NtUserDeferWindowPos', # 0x15b
	'NtUserDefSetText', # 0x15c
	'NtUserDeleteMenu', # 0x15d
	'NtUserDestroyAcceleratorTable', # 0x15e
	'NtUserDestroyCursor', # 0x15f
	'NtUserDestroyInputContext', # 0x160
	'NtUserDestroyMenu', # 0x161
	'NtUserDestroyWindow', # 0x162
	'NtUserDisableThreadIme', # 0x163
	'NtUserDispatchMessage', # 0x164
	'NtUserDragDetect', # 0x165
	'NtUserDragObject', # 0x166
	'NtUserDrawAnimatedRects', # 0x167
	'NtUserDrawCaption', # 0x168
	'NtUserDrawCaptionTemp', # 0x169
	'NtUserDrawIconEx', # 0x16a
	'NtUserDrawMenuBarTemp', # 0x16b
	'NtUserEmptyClipboard', # 0x16c
	'NtUserEnableMenuItem', # 0x16d
	'NtUserEnableScrollBar', # 0x16e
	'NtUserEndDeferWindowPosEx', # 0x16f
	'NtUserEndMenu', # 0x170
	'NtUserEndPaint', # 0x171
	'NtUserEnumDisplayDevices', # 0x172
	'NtUserEnumDisplayMonitors', # 0x173
	'NtUserEnumDisplaySettings', # 0x174
	'NtUserEvent', # 0x175
	'NtUserExcludeUpdateRgn', # 0x176
	'NtUserFillWindow', # 0x177
	'NtUserFindExistingCursorIcon', # 0x178
	'NtUserFindWindowEx', # 0x179
	'NtUserFlashWindowEx', # 0x17a
	'NtUserGetAltTabInfo', # 0x17b
	'NtUserGetAncestor', # 0x17c
	'NtUserGetAppImeLevel', # 0x17d
	'NtUserGetAsyncKeyState', # 0x17e
	'NtUserGetAtomName', # 0x17f
	'NtUserGetCaretBlinkTime', # 0x180
	'NtUserGetCaretPos', # 0x181
	'NtUserGetClassInfoEx', # 0x182
	'NtUserGetClassName', # 0x183
	'NtUserGetClipboardData', # 0x184
	'NtUserGetClipboardFormatName', # 0x185
	'NtUserGetClipboardOwner', # 0x186
	'NtUserGetClipboardSequenceNumber', # 0x187
	'NtUserGetClipboardViewer', # 0x188
	'NtUserGetClipCursor', # 0x189
	'NtUserGetComboBoxInfo', # 0x18a
	'NtUserGetControlBrush', # 0x18b
	'NtUserGetControlColor', # 0x18c
	'NtUserGetCPD', # 0x18d
	'NtUserGetCursorFrameInfo', # 0x18e
	'NtUserGetCursorInfo', # 0x18f
	'NtUserGetDC', # 0x190
	'NtUserGetDCEx', # 0x191
	'NtUserGetDoubleClickTime', # 0x192
	'NtUserGetForegroundWindow', # 0x193
	'NtUserGetGuiResources', # 0x194
	'NtUserGetGUIThreadInfo', # 0x195
	'NtUserGetIconInfo', # 0x196
	'NtUserGetIconSize', # 0x197
	'NtUserGetImeHotKey', # 0x198
	'NtUserGetImeInfoEx', # 0x199
	'NtUserGetInternalWindowPos', # 0x19a
	'NtUserGetKeyboardLayoutList', # 0x19b
	'NtUserGetKeyboardLayoutName', # 0x19c
	'NtUserGetKeyboardState', # 0x19d
	'NtUserGetKeyNameText', # 0x19e
	'NtUserGetKeyState', # 0x19f
	'NtUserGetListBoxInfo', # 0x1a0
	'NtUserGetMenuBarInfo', # 0x1a1
	'NtUserGetMenuIndex', # 0x1a2
	'NtUserGetMenuItemRect', # 0x1a3
	'NtUserGetMessage', # 0x1a4
	'NtUserGetMouseMovePointsEx', # 0x1a5
	'NtUserGetObjectInformation', # 0x1a6
	'NtUserGetOpenClipboardWindow', # 0x1a7
	'NtUserGetPriorityClipboardFormat', # 0x1a8
	'NtUserGetProcessWindowStation', # 0x1a9
	'NtUserGetRawInputBuffer', # 0x1aa
	'NtUserGetRawInputData', # 0x1ab
	'NtUserGetRawInputDeviceInfo', # 0x1ac
	'NtUserGetRawInputDeviceList', # 0x1ad
	'NtUserGetRegisteredRawInputDevices', # 0x1ae
	'NtUserGetScrollBarInfo', # 0x1af
	'NtUserGetSystemMenu', # 0x1b0
	'NtUserGetThreadDesktop', # 0x1b1
	'NtUserGetThreadState', # 0x1b2
	'NtUserGetTitleBarInfo', # 0x1b3
	'NtUserGetUpdateRect', # 0x1b4
	'NtUserGetUpdateRgn', # 0x1b5
	'NtUserGetWindowDC', # 0x1b6
	'NtUserGetWindowPlacement', # 0x1b7
	'NtUserGetWOWClass', # 0x1b8
	'NtUserHardErrorControl', # 0x1b9
	'NtUserHideCaret', # 0x1ba
	'NtUserHiliteMenuItem', # 0x1bb
	'NtUserImpersonateDdeClientWindow', # 0x1bc
	'NtUserInitialize', # 0x1bd
	'NtUserInitializeClientPfnArrays', # 0x1be
	'NtUserInitTask', # 0x1bf
	'NtUserInternalGetWindowText', # 0x1c0
	'NtUserInvalidateRect', # 0x1c1
	'NtUserInvalidateRgn', # 0x1c2
	'NtUserIsClipboardFormatAvailable', # 0x1c3
	'NtUserKillTimer', # 0x1c4
	'NtUserLoadKeyboardLayoutEx', # 0x1c5
	'NtUserLockWindowStation', # 0x1c6
	'NtUserLockWindowUpdate', # 0x1c7
	'NtUserLockWorkStation', # 0x1c8
	'NtUserMapVirtualKeyEx', # 0x1c9
	'NtUserMenuItemFromPoint', # 0x1ca
	'NtUserMessageCall', # 0x1cb
	'NtUserMinMaximize', # 0x1cc
	'NtUserMNDragLeave', # 0x1cd
	'NtUserMNDragOver', # 0x1ce
	'NtUserModifyUserStartupInfoFlags', # 0x1cf
	'NtUserMoveWindow', # 0x1d0
	'NtUserNotifyIMEStatus', # 0x1d1
	'NtUserNotifyProcessCreate', # 0x1d2
	'NtUserNotifyWinEvent', # 0x1d3
	'NtUserOpenClipboard', # 0x1d4
	'NtUserOpenDesktop', # 0x1d5
	'NtUserOpenInputDesktop', # 0x1d6
	'NtUserOpenWindowStation', # 0x1d7
	'NtUserPaintDesktop', # 0x1d8
	'NtUserPeekMessage', # 0x1d9
	'NtUserPostMessage', # 0x1da
	'NtUserPostThreadMessage', # 0x1db
	'NtUserPrintWindow', # 0x1dc
	'NtUserProcessConnect', # 0x1dd
	'NtUserQueryInformationThread', # 0x1de
	'NtUserQueryInputContext', # 0x1df
	'NtUserQuerySendMessage', # 0x1e0
	'NtUserQueryWindow', # 0x1e1
	'NtUserRealChildWindowFromPoint', # 0x1e2
	'NtUserRealInternalGetMessage', # 0x1e3
	'NtUserRealWaitMessageEx', # 0x1e4
	'NtUserRedrawWindow', # 0x1e5
	'NtUserRegisterClassExWOW', # 0x1e6
	'NtUserRegisterUserApiHook', # 0x1e7
	'NtUserRegisterHotKey', # 0x1e8
	'NtUserRegisterRawInputDevices', # 0x1e9
	'NtUserRegisterTasklist', # 0x1ea
	'NtUserRegisterWindowMessage', # 0x1eb
	'NtUserRemoveMenu', # 0x1ec
	'NtUserRemoveProp', # 0x1ed
	'NtUserResolveDesktop', # 0x1ee
	'NtUserResolveDesktopForWOW', # 0x1ef
	'NtUserSBGetParms', # 0x1f0
	'NtUserScrollDC', # 0x1f1
	'NtUserScrollWindowEx', # 0x1f2
	'NtUserSelectPalette', # 0x1f3
	'NtUserSendInput', # 0x1f4
	'NtUserSetActiveWindow', # 0x1f5
	'NtUserSetAppImeLevel', # 0x1f6
	'NtUserSetCapture', # 0x1f7
	'NtUserSetClassLong', # 0x1f8
	'NtUserSetClassWord', # 0x1f9
	'NtUserSetClipboardData', # 0x1fa
	'NtUserSetClipboardViewer', # 0x1fb
	'NtUserSetConsoleReserveKeys', # 0x1fc
	'NtUserSetCursor', # 0x1fd
	'NtUserSetCursorContents', # 0x1fe
	'NtUserSetCursorIconData', # 0x1ff
	'NtUserSetFocus', # 0x200
	'NtUserSetImeHotKey', # 0x201
	'NtUserSetImeInfoEx', # 0x202
	'NtUserSetImeOwnerWindow', # 0x203
	'NtUserSetInformationProcess', # 0x204
	'NtUserSetInformationThread', # 0x205
	'NtUserSetInternalWindowPos', # 0x206
	'NtUserSetKeyboardState', # 0x207
	'NtUserSetLogonNotifyWindow', # 0x208
	'NtUserSetMenu', # 0x209
	'NtUserSetMenuContextHelpId', # 0x20a
	'NtUserSetMenuDefaultItem', # 0x20b
	'NtUserSetMenuFlagRtoL', # 0x20c
	'NtUserSetObjectInformation', # 0x20d
	'NtUserSetParent', # 0x20e
	'NtUserSetProcessWindowStation', # 0x20f
	'NtUserSetProp', # 0x210
	'NtUserSetScrollInfo', # 0x211
	'NtUserSetShellWindowEx', # 0x212
	'NtUserSetSysColors', # 0x213
	'NtUserSetSystemCursor', # 0x214
	'NtUserSetSystemMenu', # 0x215
	'NtUserSetSystemTimer', # 0x216
	'NtUserSetThreadDesktop', # 0x217
	'NtUserSetThreadLayoutHandles', # 0x218
	'NtUserSetThreadState', # 0x219
	'NtUserSetTimer', # 0x21a
	'NtUserSetWindowFNID', # 0x21b
	'NtUserSetWindowLong', # 0x21c
	'NtUserSetWindowPlacement', # 0x21d
	'NtUserSetWindowPos', # 0x21e
	'NtUserSetWindowRgn', # 0x21f
	'NtUserSetWindowsHookAW', # 0x220
	'NtUserSetWindowsHookEx', # 0x221
	'NtUserSetWindowStationUser', # 0x222
	'NtUserSetWindowWord', # 0x223
	'NtUserSetWinEventHook', # 0x224
	'NtUserShowCaret', # 0x225
	'NtUserShowScrollBar', # 0x226
	'NtUserShowWindow', # 0x227
	'NtUserShowWindowAsync', # 0x228
	'NtUserSoundSentry', # 0x229
	'NtUserSwitchDesktop', # 0x22a
	'NtUserSystemParametersInfo', # 0x22b
	'NtUserTestForInteractiveUser', # 0x22c
	'NtUserThunkedMenuInfo', # 0x22d
	'NtUserThunkedMenuItemInfo', # 0x22e
	'NtUserToUnicodeEx', # 0x22f
	'NtUserTrackMouseEvent', # 0x230
	'NtUserTrackPopupMenuEx', # 0x231
	'NtUserCalcMenuBar', # 0x232
	'NtUserPaintMenuBar', # 0x233
	'NtUserTranslateAccelerator', # 0x234
	'NtUserTranslateMessage', # 0x235
	'NtUserUnhookWindowsHookEx', # 0x236
	'NtUserUnhookWinEvent', # 0x237
	'NtUserUnloadKeyboardLayout', # 0x238
	'NtUserUnlockWindowStation', # 0x239
	'NtUserUnregisterClass', # 0x23a
	'NtUserUnregisterUserApiHook', # 0x23b
	'NtUserUnregisterHotKey', # 0x23c
	'NtUserUpdateInputContext', # 0x23d
	'NtUserUpdateInstance', # 0x23e
	'NtUserUpdateLayeredWindow', # 0x23f
	'NtUserGetLayeredWindowAttributes', # 0x240
	'NtUserSetLayeredWindowAttributes', # 0x241
	'NtUserUpdatePerUserSystemParameters', # 0x242
	'NtUserUserHandleGrantAccess', # 0x243
	'NtUserValidateHandleSecure', # 0x244
	'NtUserValidateRect', # 0x245
	'NtUserValidateTimerCallback', # 0x246
	'NtUserVkKeyScanEx', # 0x247
	'NtUserWaitForInputIdle', # 0x248
	'NtUserWaitForMsgAndEvent', # 0x249
	'NtUserWaitMessage', # 0x24a
	'NtUserWin32PoolAllocationStats', # 0x24b
	'NtUserWindowFromPoint', # 0x24c
	'NtUserYieldTask', # 0x24d
	'NtUserRemoteConnect', # 0x24e
	'NtUserRemoteRedrawRectangle', # 0x24f
	'NtUserRemoteRedrawScreen', # 0x250
	'NtUserRemoteStopScreenUpdates', # 0x251
	'NtUserCtxDisplayIOCtl', # 0x252
	'NtGdiEngAssociateSurface', # 0x253
	'NtGdiEngCreateBitmap', # 0x254
	'NtGdiEngCreateDeviceSurface', # 0x255
	'NtGdiEngCreateDeviceBitmap', # 0x256
	'NtGdiEngCreatePalette', # 0x257
	'NtGdiEngComputeGlyphSet', # 0x258
	'NtGdiEngCopyBits', # 0x259
	'NtGdiEngDeletePalette', # 0x25a
	'NtGdiEngDeleteSurface', # 0x25b
	'NtGdiEngEraseSurface', # 0x25c
	'NtGdiEngUnlockSurface', # 0x25d
	'NtGdiEngLockSurface', # 0x25e
	'NtGdiEngBitBlt', # 0x25f
	'NtGdiEngStretchBlt', # 0x260
	'NtGdiEngPlgBlt', # 0x261
	'NtGdiEngMarkBandingSurface', # 0x262
	'NtGdiEngStrokePath', # 0x263
	'NtGdiEngFillPath', # 0x264
	'NtGdiEngStrokeAndFillPath', # 0x265
	'NtGdiEngPaint', # 0x266
	'NtGdiEngLineTo', # 0x267
	'NtGdiEngAlphaBlend', # 0x268
	'NtGdiEngGradientFill', # 0x269
	'NtGdiEngTransparentBlt', # 0x26a
	'NtGdiEngTextOut', # 0x26b
	'NtGdiEngStretchBltROP', # 0x26c
	'NtGdiXLATEOBJ_cGetPalette', # 0x26d
	'NtGdiXLATEOBJ_iXlate', # 0x26e
	'NtGdiXLATEOBJ_hGetColorTransform', # 0x26f
	'NtGdiCLIPOBJ_bEnum', # 0x270
	'NtGdiCLIPOBJ_cEnumStart', # 0x271
	'NtGdiCLIPOBJ_ppoGetPath', # 0x272
	'NtGdiEngDeletePath', # 0x273
	'NtGdiEngCreateClip', # 0x274
	'NtGdiEngDeleteClip', # 0x275
	'NtGdiBRUSHOBJ_ulGetBrushColor', # 0x276
	'NtGdiBRUSHOBJ_pvAllocRbrush', # 0x277
	'NtGdiBRUSHOBJ_pvGetRbrush', # 0x278
	'NtGdiBRUSHOBJ_hGetColorTransform', # 0x279
	'NtGdiXFORMOBJ_bApplyXform', # 0x27a
	'NtGdiXFORMOBJ_iGetXform', # 0x27b
	'NtGdiFONTOBJ_vGetInfo', # 0x27c
	'NtGdiFONTOBJ_pxoGetXform', # 0x27d
	'NtGdiFONTOBJ_cGetGlyphs', # 0x27e
	'NtGdiFONTOBJ_pifi', # 0x27f
	'NtGdiFONTOBJ_pfdg', # 0x280
	'NtGdiFONTOBJ_pQueryGlyphAttrs', # 0x281
	'NtGdiFONTOBJ_pvTrueTypeFontFile', # 0x282
	'NtGdiFONTOBJ_cGetAllGlyphHandles', # 0x283
	'NtGdiSTROBJ_bEnum', # 0x284
	'NtGdiSTROBJ_bEnumPositionsOnly', # 0x285
	'NtGdiSTROBJ_bGetAdvanceWidths', # 0x286
	'NtGdiSTROBJ_vEnumStart', # 0x287
	'NtGdiSTROBJ_dwGetCodePage', # 0x288
	'NtGdiPATHOBJ_vGetBounds', # 0x289
	'NtGdiPATHOBJ_bEnum', # 0x28a
	'NtGdiPATHOBJ_vEnumStart', # 0x28b
	'NtGdiPATHOBJ_vEnumStartClipLines', # 0x28c
	'NtGdiPATHOBJ_bEnumClipLines', # 0x28d
	'NtGdiGetDhpdev', # 0x28e
	'NtGdiEngCheckAbort', # 0x28f
	'NtGdiHT_Get8BPPFormatPalette', # 0x290
	'NtGdiHT_Get8BPPMaskPalette', # 0x291
	'NtGdiUpdateTransform', # 0x292
	'NtGdiSetPUMPDOBJ', # 0x293
	'NtGdiBRUSHOBJ_DeleteRbrush', # 0x294
	'NtGdiUMPDEngFreeUserMem', # 0x295
	'NtGdiDrawStream', # 0x296
	'NtGdiMakeObjectXferable', # 0x297
	'DxEngGetRedirectionBitmap', # 0x298
	],
]



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.utils as utils
import volatility.plugins.gui.atoms as atoms
import volatility.plugins.gui.constants as consts
import volatility.plugins.gui.sessions as sessions

# Offsets to (_catomSysTableEntries, _aatomSysLoaded) in win32k.sys. We use
# this for translating the ihmod value into a fully-qualified DLL path name 
# used by messagehooks and eventhooks plugins. If the values for your system
# aren't in the list, the plugins will still work, but the names of the Hook
# Module will not be available.
message_offsets_x86 = [
      (0x001ab0a0, 0x001ab060), # ? (shylock.dmp)
      (0x001aaea0, 0x001aae60), # 5.1.2600.6033 (XP SP3)
      (0x001ac640, 0x001ac600), # 5.1.2600.6149 (XP)
      (0x001a9400, 0x001a93c0), # 5.1.2600.5512 (XP SP3)
      (0x001a9220, 0x001a91e0), # 5.1.2600.3335 (XP SP2) 
      (0x001a6f00, 0x001a6ec0), # 5.1.2600.2180 (XP SP2)    
      (0x001a0338, 0x001a03c0), # ? (W2K3 SP0)
      (0x001b5600, 0x001b55c0), # 5.2.3790.4980 (W2K3 SP2)
      (0x001b1440, 0x001b1400), # 5.2.3790.1830 (W2K3 SP1)
      (0x001de0e0, 0x001de0a0), # 6.0.6000.16386 (Vista SP0)
      (0x001e01e0, 0x001e01a0), # 6.0.6002.18005 (Vista SP2)
      (0x001df0e0, 0x001df0a0), # 6.0.6001.18000 (W2K8 SP1)
      (0x00219800, 0x002197C0), # 6.1.7600.16385 (Win 7 SP0)
      (0x0021e800, 0x0021e7c0), # 6.1.7600.16988 (Win 7 SP0)
      (0x0021a900, 0x0021a8c0), # 6.1.7601.17514 (Win 7 SP1)
      ]
message_offsets_x64 = [
      (0x003b3880, 0x003b3840), # 5.2.3790.1830 (W2K3 SP1 / XP SP1)
      (0x003b4880, 0x003b4840), # 5.2.3790.3959 (W2K3 SP2 / XP SP2)
      (0x0028ba20, 0x0028b9e0), # 6.0.6000.16386 (Vista SP0)
      (0x00288a20, 0x002889e0), # 6.0.6001.18000 (Vista SP1 / W2K8 SP1)
      (0x00289c20, 0x00289be0), # 6.0.6002.18005 (Vista SP2 / W2K8 SP2)
      (0x002da480, 0x002da440), # 6.1.7600.16385 (Win 7 SP0)
      (0x002db6a0, 0x002db660), # 6.1.7601.17514 (Win 7 SP1)
      (0x002e08a0, 0x002e0860), # 6.1.7601.17842 (W2K8 R2 SP1)
      (0x002e06a0, 0x002e0660), # ?? (W2K8 R2 SP1)
      ]

class MessageHooks(atoms.Atoms, sessions.SessionsMixin):
    """List desktop and thread window message hooks"""

    def calculate(self):
        # Get all the atom tables and window stations 
        atom_tables = dict((atom_table, winsta)
            for (atom_table, winsta)
            in atoms.Atoms(self._config).calculate())

        # Unique window stations
        window_stations = [
                winsta for winsta in atom_tables.values()
                if winsta]

        for winsta in window_stations:
            yield winsta, atom_tables

    def translate_atom(self, winsta, atom_tables, atom_id):
        """
        Translate an atom into an atom name.

        @param winsta: a tagWINDOWSTATION in the proper 
        session space 

        @param atom_tables: a dictionary with _RTL_ATOM_TABLE
        instances as the keys and owning window stations as
        the values. 

        @param index: the index into the atom handle table. 
        """

        # First check the default atoms
        if consts.DEFAULT_ATOMS.has_key(atom_id):
            return consts.DEFAULT_ATOMS[atom_id].Name

        # A list of tables to search. The session atom tables
        # have priority and will be searched first. 
        table_list = [
                table for (table, window_station)
                in atom_tables.items() if window_station == None
                ]
        table_list.append(winsta.AtomTable)

        ## Fixme: the session atom tables are found via physical
        ## AS pool tag scanning, and there's no good way (afaik)
        ## to associate the table with its session. Thus if more
        ## than one session has atoms with the same id but different
        ## values, then we could possibly select the wrong one. 
        for table in table_list:
            atom = table.find_atom(atom_id)
            if atom:
                return atom.Name

        return obj.NoneObject("Cannot translate atom {0:#x}".format(atom_id))

    def translate_hmod(self, winsta, atom_tables, index):
        """
        Translate an ihmod (index into a handle table) into
        an atom. This requires locating the win32k!_aatomSysLoaded 
        symbol. If the  symbol cannot be found, we'll just report 
        back the ihmod value. 

        @param winsta: a tagWINDOWSTATION in the proper 
        session space 

        @param atom_tables: a dictionary with _RTL_ATOM_TABLE
        instances as the keys and owning window stations as
        the values. 

        @param index: the index into the atom handle table. 
        """

        # No need to translate these
        if index == -1:
            return "(Current Module)"

        # To get an _MM_SESSION_SPACE we first start with a 
        # kernel AS and walk processes. 
        kernel_space = utils.load_as(self._config)

        session = self.find_session_space(
                kernel_space, winsta.dwSessionId)

        # Report back the ihmod value if we fail 
        if not session:
            return hex(index)

        if winsta.obj_vm.profile.metadata.get('memory_model', '32bit') == '32bit':
            message_offsets = message_offsets_x86
        else:
            message_offsets = message_offsets_x64

        # Iterate over the possible offsets for win32k globals 
        for (count_offset, table_offset) in message_offsets:

            # This is _catomSysTableEntries
            count = obj.Object("unsigned long",
                            offset = session.Win32KBase + count_offset,
                            vm = session.obj_vm)

            # We fail for this offset if the count is unreadable, 
            # its greater than 32, or its less than the requested
            # handle table index. 
            if (count == None or count == 0 or count > 32 or
                    count <= index):
                continue

            # An array of atom IDs
            atomlist = obj.Object("Array", targetType = "unsigned short",
                offset = session.Win32KBase + table_offset,
                count = count, vm = session.obj_vm)

            # Our last sanity check is that the number of valid 
            # atoms equals the claimed number of atoms. This check 
            # is currently commented out because on at least one image
            # (shylock.dmp), the count is 3 but there are only 2 valid
            # atoms, thus we end up skipping it. 
            #valid_entries = len([atom for atom in atoms if atom != 0])
            #if count != valid_entries:
            #    continue

            # We can stop after finding a potential atom 
            atom_id = atomlist[index]

            # Attempt to translate the atom into a module name
            module = self.translate_atom(winsta, atom_tables, atom_id)
            if module:
                return module

        # Report back the ihmod value if we fail 
        return hex(index)

    def render_text(self, outfd, data):
        """Render output in table form"""

        self.table_header(outfd,
                        [("Offset(V)", "[addrpad]"),
                         ("Sess", "<6"),
                         ("Desktop", "20"),
                         ("Thread", "30"),
                         ("Filter", "20"),
                         ("Flags", "20"),
                         ("Function", "[addrpad]"),
                         ("Module", ""),
                        ])

        for winsta, atom_tables in data:
            for desk in winsta.desktops():
                for name, hook in desk.hooks():
                    module = self.translate_hmod(winsta, atom_tables, hook.ihmod)
                    self.table_row(outfd,
                            hook.obj_offset,
                            winsta.dwSessionId,
                            "{0}\\{1}".format(winsta.Name, desk.Name),
                            "<any>", name,
                            str(hook.flags),
                            hook.offPfn,
                            module,
                            )

                for thrd in desk.threads():
                    info = "{0} ({1} {2})".format(
                            thrd.pEThread.Cid.UniqueThread,
                            thrd.ppi.Process.ImageFileName,
                            thrd.ppi.Process.UniqueProcessId
                            )
                    for name, hook in thrd.hooks():
                        module = self.translate_hmod(winsta, atom_tables, hook.ihmod)
                        self.table_row(outfd,
                                    hook.obj_offset,
                                    winsta.dwSessionId,
                                    "{0}\\{1}".format(winsta.Name, desk.Name),
                                    info, name,
                                    str(hook.flags),
                                    hook.offPfn,
                                    module,
                                    )

    def render_block(self, outfd, data):
        """Render output as a block"""

        def write_block(outfd, winsta, desk, hook, module, thread):
            outfd.write("{0:<10} : {1:#x}\n".format("Offset(V)", hook.obj_offset))
            outfd.write("{0:<10} : {1}\n".format("Session", winsta.dwSessionId))
            outfd.write("{0:<10} : {1}\n".format("Desktop", "{0}\\{1}".format(winsta.Name, desk.Name)))
            outfd.write("{0:<10} : {1}\n".format("Thread", thread))
            outfd.write("{0:<10} : {1}\n".format("Filter", name))
            outfd.write("{0:<10} : {1}\n".format("Flags", str(hook.flags)))
            outfd.write("{0:<10} : {1:#x}\n".format("Procedure", hook.offPfn))
            outfd.write("{0:<10} : {1}\n".format("ihmod", hook.ihmod))
            outfd.write("{0:<10} : {1}\n\n".format("Module", module))

        for winsta, atom_tables in data:
            for desk in winsta.desktops():
                for name, hook in desk.hooks():
                    module = self.translate_hmod(winsta, atom_tables, hook.ihmod)
                    write_block(outfd, winsta, desk, hook, module, "<any>")

                for thrd in desk.threads():
                    info = "{0} ({1} {2})".format(
                            thrd.pEThread.Cid.UniqueThread,
                            thrd.ppi.Process.ImageFileName,
                            thrd.ppi.Process.UniqueProcessId
                            )
                    for name, hook in thrd.hooks():
                        module = self.translate_hmod(winsta, atom_tables, hook.ihmod)
                        write_block(outfd, winsta, desk, hook, module, info)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.pstasks as pstasks
import volatility.plugins.mac.common as common

class mac_lsof(pstasks.mac_tasks):
    """ Lists per-process opened files """
    def calculate(self):
        common.set_plugin_members(self)

        procs = pstasks.mac_tasks(self._config).calculate()

        for proc in procs:
            fds = obj.Object('Array', offset = proc.p_fd.fd_ofiles, vm = self.addr_space, targetType = 'Pointer', count = proc.p_fd.fd_lastfile)

            for i, fd in enumerate(fds):
                f = fd.dereference_as("fileproc")
                if f:
                    ## FIXME after 2.3 replace this explicit int field with the following line:
                    ##    if str(f.f_fglob.fg_type) == 'DTYPE_VNODE':
                    ## Its not needed for profiles generated with convert.py after r3290 
                    fg_type = obj.Object("int", f.f_fglob.fg_type.obj_offset, vm = self.addr_space)
                    if fg_type == 1: # VNODE
                        vnode = f.f_fglob.fg_data.dereference_as("vnode")
                        path = vnode.full_path()
                    else:
                        path = ""
                                        
                    yield proc, i, f, path
 
    def render_text(self, outfd, data):
        self.table_header(outfd, [("PID","8"),
                                  ("File Descriptor", "6"),
                                  ("File Path", ""),
                                 ])
 
        for proc, i, f, path in data:
            if path:
                self.table_row(outfd, proc.p_pid, i, path)

   



import volatility.conf as conf
import volatility.constants as constants
import os
import sys

config = conf.ConfObject()

help_prefix = ""
plugin_separator = ":"
# Make a platform-dependent decision on plugin path separators
# The separator is now in keeping with the PATH environment variable
if sys.platform.startswith('win'):
    help_prefix = "semi-"
    plugin_separator = ";"


config.add_option("PLUGINS", default = "",
                  cache_invalidator = False,
                  help = "Additional plugin directories to use (" + help_prefix + "colon separated)")

# Add the PLUGINPATH, in case we're frozen
__path__ = [constants.PLUGINPATH] + [ e for e in __path__ if not constants.PLUGINPATH.startswith(e) ]

# This causes the config.PLUGINS paths to be treated as extensions of the volatility.plugins package
# Meaning that each directory is search for module when import volatility.plugins.module is requested

if config.PLUGINS:
    plugin_paths = [ os.path.abspath(x) for x in config.PLUGINS.split(plugin_separator)]
    __path__.extend(plugin_paths)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.debug as debug
import volatility.utils as utils
import volatility.plugins.common as common
import volatility.plugins.gui.sessions as sessions
import volatility.plugins.gui.windowstations as windowstations
import volatility.plugins.gui.constants as consts

class Clipboard(common.AbstractWindowsCommand, sessions.SessionsMixin):
    """Extract the contents of the windows clipboard"""

    def calculate(self):
        kernel_space = utils.load_as(self._config)

        # Dictionary of MM_SESSION_SPACEs by ID
        sesses = dict((int(session.SessionId), session)
            for session in self.session_spaces(kernel_space)
                )

        # Dictionary of session USER objects by handle
        session_handles = {}

        # If various objects cannot be found or associated, 
        # we'll return none objects
        e0 = obj.NoneObject("Unknown tagCLIPDATA")
        e1 = obj.NoneObject("Unknown tagWINDOWSTATION")
        e2 = obj.NoneObject("Unknown tagCLIP")

        # Handle type filter 
        filters = [lambda x : str(x.bType) == "TYPE_CLIPDATA"]

        # Load tagCLIPDATA handles from all sessions 
        for sid, session in sesses.items():
            handles = {}
            shared_info = session.find_shared_info()
            if not shared_info:
                debug.debug("No shared info for session {0}".format(sid))
                continue
            for handle in shared_info.handles(filters):
                handles[int(handle.phead.h)] = handle
            session_handles[sid] = handles

        # Each WindowStation 
        for wndsta in windowstations.WndScan(self._config).calculate():
            session = sesses.get(int(wndsta.dwSessionId), None)
            # The session is unknown 
            if not session:
                continue
            handles = session_handles.get(int(session.SessionId), None)
            # No handles in the session 
            if not handles:
                continue
            clip_array = wndsta.pClipBase.dereference()
            # The tagCLIP array is empty or the pointer is invalid 
            if not clip_array:
                continue
            # Resolve tagCLIPDATA from tagCLIP.hData 
            for clip in clip_array:
                handle = handles.get(int(clip.hData), e0)
                # Remove this handle from the list 
                if handle:
                    handles.pop(int(clip.hData))
                yield session, wndsta, clip, handle

        # Any remaining tagCLIPDATA not matched. This allows us
        # to still find clipboard data if a window station is not
        # found or if pClipData or cNumClipFormats were corrupt
        for sid in sesses.keys():
            handles = session_handles.get(sid, None)
            # No handles in the session 
            if not handles:
                continue
            for handle in handles.values():
                yield sesses[sid], e1, e2, handle

    def render_text(self, outfd, data):

        self.table_header(outfd,
                         [("Session", "10"),
                          ("WindowStation", "12"),
                          ("Format", "18"),
                          ("Handle", "[addr]"),
                          ("Object", "[addrpad]"),
                          ("Data", "50"),
                         ])

        for session, wndsta, clip, handle in data:

            # If no tagCLIP is provided, we do not know the format
            if not clip:
                fmt = obj.NoneObject("Format unknown")
            else:
                # Try to get the format name, but failing that, print 
                # the format number in hex instead. 
                if clip.fmt.v() in consts.CLIPBOARD_FORMAT_ENUM:
                    fmt = str(clip.fmt)
                else:
                    fmt = hex(clip.fmt.v())

            # Try to get the handle from tagCLIP first, but
            # fall back to using _HANDLEENTRY.phead. Note: this can
            # be a value like DUMMY_TEXT_HANDLE (1) etc.
            if clip:
                handle_value = clip.hData
            else:
                handle_value = handle.phead.h

            clip_data = ""
            if handle and "TEXT" in fmt:
                clip_data = handle.reference_object().as_string(fmt)

            self.table_row(outfd,
                           session.SessionId,
                           wndsta.Name,
                           fmt,
                           handle_value,
                           handle.phead.v(),
                           clip_data)

            # Print an additional hexdump if --verbose is specified
            if self._config.VERBOSE and handle:
                hex_dump = handle.reference_object().as_hex()
                outfd.write("{0}".format(hex_dump))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import os.path
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.proc_maps as linux_proc_maps

class linux_dump_map(linux_proc_maps.linux_proc_maps):
    """ Writes selected memory mappings to disk """

    def __init__(self, config, *args, **kwargs):
        linux_proc_maps.linux_proc_maps.__init__(self, config, *args, **kwargs)
        self._config.add_option('VMA', short_option = 's', default = None, help = 'Filter by VMA starting address', action = 'store', type = 'long')
        self._config.add_option('DUMP-DIR', short_option = 'D', default = None, help = 'Output directory', action = 'store', type = 'str')

    def read_addr_range(self, task, start, end):
        pagesize = 4096 

        # set the as with our new dtb so we can read from userland
        proc_as = task.get_process_address_space()

        # xrange doesn't support longs :(
        while start < end:
            page = proc_as.zread(start, pagesize)
            yield page
            start = start + pagesize

    def render_text(self, outfd, data):
        if (not self._config.DUMP_DIR or 
                not os.path.isdir(self._config.DUMP_DIR)):
            debug.error("Please specify an existing output dir (--dump-dir)")

        self.table_header(outfd, [("Task", "10"), 
                                  ("VM Start", "[addrpad]"), 
                                  ("VM End", "[addrpad]"), 
                                  ("Length", "[addr]"), 
                                  ("Path", "")])

        for (task, vma) in data:
            if not self._config.VMA or vma.vm_start == self._config.VMA:
                file_name = "task.{0}.{1:#x}.vma".format(task.pid, vma.vm_start)
                file_path = os.path.join(self._config.DUMP_DIR, file_name)
                
                outfile = open(file_path, "wb+")
                for page in self.read_addr_range(task, vma.vm_start, vma.vm_end):
                    outfile.write(page)
                outfile.close()
                
                self.table_row(outfd, task.pid, 
                               vma.vm_start, 
                               vma.vm_end, 
                               vma.vm_end - vma.vm_start, 
                               file_path)




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj as obj
import volatility.plugins.mac.pslist as pslist
import volatility.plugins.mac.common as common

class mac_tasks(pslist.mac_pslist):
    """ List Active Tasks """
    def __init__(self, config, *args, **kwargs):
        pslist.mac_pslist.__init__(self, config, *args, **kwargs)

    def calculate(self):
        common.set_plugin_members(self)
        
        pidlist = None
        try:
            if self._config.PID:
                pidlist = [int(p) for p in self._config.PID.split(',')]
        except:
            pass
        
        tasksaddr = self.addr_space.profile.get_symbol("_tasks")
        queue_entry = obj.Object("queue_entry", offset = tasksaddr, vm = self.addr_space)

        seen = [tasksaddr]

        for task in queue_entry.walk_list(list_head = tasksaddr):
            if (task.bsd_info and task.obj_offset not in seen):
                proc = task.bsd_info.dereference_as("proc") 
                if not pidlist or proc.p_pid in pidlist:
                    yield proc 
                seen.append(task.obj_offset)



## This file was taken from PyFlag http://www.pyflag.net/
# Michael Cohen <scudette@users.sourceforge.net>
# David Collett <daveco@users.sourceforge.net>
#
# ******************************************************
#  Version: FLAG $Version: 0.87-pre1 Date: Thu Jun 12 00:48:38 EST 2008$
# ******************************************************
#
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License
# * as published by the Free Software Foundation; either version 2
# * of the License, or (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ******************************************************

#pylint: disable-msg=C0111

""" Configuration modules for pyflag.

PyFlag is a complex package and requires a flexible configuration
system. The following are the requirements of the configuration
system:

1) Configuration must be available from a number of sources:

   - Autoconf must be able to set things like the python path (in case
     pyflag is installed to a different prefix)
     
   - Users must be able to configure the installed system for their
   specific requirements.

   - Unconfigured parameters must be resolved at run time through the
   GUI and saved.

2) Configuration must be able to apply to cases specifically.

3) Because pyflag is modular, configuration variables might be required
   for each module. This means that definitions and declarations of
   configuration variables must be distributed in each plugin.

These goals are achieved by the use of multiple sources of
configuration information:

   - The system wide configuration file is this file: conf.py. It is
   generated from the build system from conf.py.in by substituting
   autoconfigured variables into it. It contains the most basic
   settings related to the installation, e.g. which python interpreted
   is used, where the python modules are installed etc. In particular
   it refers to the location of the system configuration file (usually
   found in /usr/local/etc/pyflagrc, or in /etc/pyflagrc).

   - The sysconfig file contains things like where the upload
   directory is, where to store temporary files etc. These are mainly
   installation wide settings which are expected to be modified by the
   administrator. Note that if you want the GUI to manipulate this
   file it needs to be writable by the user running the GUI.

   - Finally a conf table in each case is used to provide a per case
   configuration
   
"""
import ConfigParser
import optparse
import os
import sys

default_config = "/etc/volatilityrc"

class PyFlagOptionParser(optparse.OptionParser):
    final = False
    help_hooks = []

    def _process_args(self, largs, rargs, values):
        try:
            return optparse.OptionParser._process_args(self, largs, rargs, values)
        except (optparse.BadOptionError, optparse.OptionValueError), err:
            if self.final:
                raise err

    def error(self, msg):
        ## We cant emit errors about missing parameters until we are
        ## sure that all modules have registered all their parameters
        if self.final:
            return optparse.OptionParser.error(self, msg)
        else:
            raise RuntimeError(msg)

    def print_help(self, file = sys.stdout):
        optparse.OptionParser.print_help(self, file)

        for cb in self.help_hooks:
            file.write(cb())

class ConfObject(object):
    """ This is a singleton class to manage the configuration.

    This means it can be instantiated many times, but each instance
    refers to the global configuration (which is set in class
    variables).

    NOTE: The class attributes have static dicts assigned to
    facilitate singleton behaviour. This means all future instances
    will have the same dicts.
    """
    optparser = PyFlagOptionParser(add_help_option = False,
                                   version = False,
                                   )
    initialised = False

    ## This is the globals dictionary which will be used for
    ## evaluating the configuration directives.
    g_dict = dict(__builtins__ = None)

    ## These are the options derived by reading any config files
    cnf_opts = {}

    ## Command line opts
    opts = {}
    args = None
    default_opts = {}
    docstrings = {}

    ## These are the actual options returned by the optparser:
    optparse_opts = None

    ## Filename where the configuration file is:
    _filename = None

    _filenames = []

    ## These parameters can not be updated by the GUI (but will be
    ## propagated into new configuration files)
    readonly = {}

    ## Absolute parameters can only be set by the code or command
    ## lines, they can not be over ridden in the configuration
    ## file. This ensures that only configuration files dont mask new
    ## options (e.g. schema version)
    _absolute = {}

    ## A list of option names:
    options = []

    ## Cache variants: There are configuration options which
    ## encapsulate the state of the running program. If any of these
    ## change all caches will be invalidated.
    cache_invalidators = {}

    def __init__(self):
        """ This is a singleton object kept in the class """
        if not ConfObject.initialised:
            self.optparser.add_option("-h", "--help", action = "store_true", default = False,
                            help = "list all available options and their default values. Default values may be set in the configuration file (" + default_config + ")")

            ConfObject.initialised = True

    def set_usage(self, usage = None, version = None):
        if usage:
            self.optparser.set_usage(usage)

        if version:
            self.optparser.version = version

    def add_file(self, filename, _type = 'init'):
        """ Adds a new file to parse """
        self._filenames.append(filename)

        self.cnf_opts.clear()

        for f in self._filenames:
            try:
                conf_parser = ConfigParser.ConfigParser()
                conf_parser.read(f)

                for k, v in conf_parser.items('DEFAULT'):
                    ## Absolute parameters are protected from
                    ## configuration files:
                    if k in self._absolute.keys():
                        continue

                    try:
                        v = eval(v, self.g_dict)
                    except Exception, _e:
                        pass

                    ## update the configured options
                    self.cnf_opts[k] = v

            except IOError:
                print "Unable to open {0}".format(f)

        ConfObject._filename = filename

    def print_help(self):
        return self.optparser.print_help()

    def add_help_hook(self, cb):
        """ Adds an epilog to the help message """
        self.optparser.help_hooks.append(cb)

    def set_help_hook(self, cb):
        self.optparser.help_hooks = [cb]

    def parse_options(self, final = True):
        """ Parses the options from command line and any conf files
        currently added.

        The final parameter should be only called from main programs
        at the point where they are prepared for us to call exit if
        required; (For example when we detect the -h parameter).
        """
        self.optparser.final = final

        ## Parse the command line options:
        try:
            (opts, args) = self.optparser.parse_args()

            self.opts.clear()

            ## Update our cmdline dict:
            for k in dir(opts):
                v = getattr(opts, k)
                if k in self.options and not v == None:
                    self.opts[k] = v

        except UnboundLocalError:
            raise RuntimeError("Unknown option - use -h to see help")

        ## If error() was called we catch it here
        except RuntimeError:
            opts = {}
            ## This gives us as much as was parsed so far
            args = self.optparser.largs

        self.optparse_opts = opts
        self.args = args

        if final:
            ## Reparse the config file again:
            self.add_file(self._filename)

            try:
                ## Help can only be set on the command line
                if getattr(self.optparse_opts, "help"):

                ## Populate the metavars with the default values:
                    for opt in self.optparser.option_list:
                        try:
                            opt.metavar = "{0}".format((getattr(self, opt.dest) or
                                                        opt.dest.upper()))
                        except Exception, _e:
                            pass

                    self.optparser.print_help()
                    sys.exit(0)
            except AttributeError:
                pass

            ## Set the cache invalidators on the cache now:
            import volatility.cache as cache
            for k, v in self.cache_invalidators.items():
                cache.CACHE.invalidate_on(k, v)

    def remove_option(self, option):
        """ Removes options both from the config file parser and the
            command line parser

            This should only by used on options *before* they have been read,
            otherwise things could get very confusing.
        """
        option = option.lower()

        if option in self.cache_invalidators:
            del self.cache_invalidators[option]

        normalized_option = option.replace("-", "_")

        if normalized_option not in self.options:
            return

        self.options.remove(normalized_option)

        if normalized_option in self.readonly:
            del self.readonly[normalized_option]

        if normalized_option in self.default_opts:
            del self.default_opts[normalized_option]

        if normalized_option in self._absolute:
            del self._absolute[normalized_option]

        del self.docstrings[normalized_option]

        self.optparser.remove_option("--{0}".format(option))

        try:
            self.parse_options(False)
        except AttributeError:
            pass

    def add_option(self, option, short_option = None,
                   cache_invalidator = True,
                   **args):
        """ Adds options both to the config file parser and the
        command line parser.

        Args:
          option:            The long option name.
          short_option:      An optional short option.
          cache_invalidator: If set, when this option
                             changes all caches are invalidated.
        """
        option = option.lower()

        if cache_invalidator:
            self.cache_invalidators[option] = lambda : self.get_value(option)

        normalized_option = option.replace("-", "_")

        if normalized_option in self.options:
            return

        self.options.append(normalized_option)

        ## If this is read only we store it in a special dict
        try:
            if args['readonly']:
                self.readonly[normalized_option] = args['default']
            del args['readonly']
        except KeyError:
            pass

        ## If there is a default specified, we update our defaults dict:
        try:
            default = args['default']
            try:
                default = eval(default, self.g_dict)
            except:
                pass

            self.default_opts[normalized_option] = default
            del args['default']
        except KeyError:
            pass

        try:
            self._absolute[normalized_option] = args['absolute']
            del args['absolute']
        except KeyError:
            pass

        self.docstrings[normalized_option] = args.get('help', None)

        if short_option:
            self.optparser.add_option("-{0}".format(short_option), "--{0}".format(option), **args)
        else:
            self.optparser.add_option("--{0}".format(option), **args)

        ## update the command line parser

        ## We have to do the try-catch for python 2.4 support of short
        ## arguments. It can be removed when python 2.5 is a requirement
        try:
            self.parse_options(False)
        except AttributeError:
            pass

    def update(self, key, value):
        """ This can be used by scripts to force a value of an option """
        self.readonly[key.lower()] = value

    def get_value(self, key):
        return getattr(self, key.replace("-", "_"))

    def __getattr__(self, attr):
        ## If someone is looking for a configuration parameter but
        ## we have not parsed anything yet - do so now.
        if self.opts == None:
            self.parse_options(False)

        ## Maybe its a class method?
        try:
            return super(ConfObject, self).__getattribute__(attr)
        except AttributeError:
            pass

        ## Is it a ready only parameter (i.e. can not be overridden by
        ## the config file)
        try:
            return self.readonly[attr.lower()]
        except KeyError:
            pass

        ## Try to find the attribute in the command line options:
        try:
            return self.opts[attr.lower()]
        except KeyError:
            pass

        ## Has it already been parsed?
        try:
            tmp = getattr(self.optparser.values, attr.lower())
            if tmp:
                return tmp
        except AttributeError:
            pass

        ## Was it given in the environment?
        try:
            return os.environ["VOLATILITY_" + attr.upper()]
        except KeyError:
            pass

        ## No - try the configuration file:
        try:
            return self.cnf_opts[attr.lower()]
        except KeyError:
            pass

        ## No - is there a default for it?
        try:
            return self.default_opts[attr.lower()]
        except KeyError:
            pass

        ## Maybe its just a command line option:
        try:
            if not attr.startswith("_") and self.optparse_opts:
                return getattr(self.optparse_opts, attr.lower())
        except AttributeError:
            pass

        raise AttributeError("Parameter {0} is not configured - try setting it on the command line (-h for help)".format(attr))

class DummyConfig(ConfObject):
    pass

config = ConfObject()
if os.access(default_config, os.R_OK):
    config.add_file(default_config)
else:
    config.add_file("volatilityrc")

default_conf_path = ".volatilityrc"
try:
    default_conf_path = os.environ['HOME'] + '/.volatilityrc'
except KeyError:
    pass

config.add_option("CONF-FILE", default = default_conf_path,
                  cache_invalidator = False,
                  help = "User based configuration file")

config.add_file(config.CONF_FILE)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.commands as commands
import volatility.utils as utils
import volatility.debug as debug
import volatility.obj as obj

MAX_STRING_LENGTH = 256

nsecs_per = 1000000000

class vol_timespec:

    def __init__(self, secs, nsecs):
        self.tv_sec  = secs
        self.tv_nsec = nsecs

def set_plugin_members(obj_ref):
    obj_ref.addr_space = utils.load_as(obj_ref._config)

    if not obj_ref.is_valid_profile(obj_ref.addr_space.profile):
        debug.error("This command does not support the selected profile.")

class AbstractLinuxCommand(commands.Command):
    def __init__(self, *args, **kwargs):
        self.addr_space = None
        self.known_addrs = {}
        commands.Command.__init__(self, *args, **kwargs)

    @property
    def profile(self):
        if self.addr_space:
            return self.addr_space.profile
        return None

    def execute(self, *args, **kwargs):
        commands.Command.execute(self, *args, **kwargs)

    @staticmethod
    def is_valid_profile(profile):
        return profile.metadata.get('os', 'Unknown').lower() == 'linux'

    def is_known_address(self, addr, modules):

        text = self.profile.get_symbol("_text")
        etext = self.profile.get_symbol("_etext")

        return (self.addr_space.address_compare(addr, text) != -1 and self.addr_space.address_compare(addr, etext) == -1) or self.address_in_module(addr, modules)

    def address_in_module(self, addr, modules):
    
        for (_, start, end) in modules:
            if self.addr_space.address_compare(addr, start) != -1 and self.addr_space.address_compare(addr, end) == -1:
                return True
    
        return False

    def verify_ops(self, ops, op_members, modules):

        for check in op_members:
            addr = ops.m(check)

            if addr and addr != 0:

                if addr in self.known_addrs:
                    known = self.known_addrs[addr]
                else:
                    known = self.is_known_address(addr, modules)
                    self.known_addrs[addr] = known
                
                if known == 0:
                    yield (check, addr)

class AbstractLinuxIntelCommand(AbstractLinuxCommand):
    @staticmethod
    def is_valid_profile(profile):
        return AbstractLinuxCommand.is_valid_profile(profile) \
        and (profile.metadata.get('arch').lower() == 'x86' \
        or profile.metadata.get('arch').lower() == 'x64')

class AbstractLinuxARMCommand(AbstractLinuxCommand):
    @staticmethod
    def is_valid_profile(profile):
        return AbstractLinuxCommand.is_valid_profile(profile) \
        and (profile.metadata.get('arch').lower() == 'arm')                   
 
def walk_internal_list(struct_name, list_member, list_start, addr_space = None):
    if not addr_space:
        addr_space = list_start.obj_vm

    while list_start:
        list_struct = obj.Object(struct_name, vm = addr_space, offset = list_start.v())
        yield list_struct
        list_start = getattr(list_struct, list_member)

# based on __d_path
def do_get_path(rdentry, rmnt, dentry, vfsmnt):
    ret_path = []

    inode = dentry.d_inode

    if not rdentry.is_valid() or not dentry.is_valid():
        return []

    while (dentry != rdentry or vfsmnt != rmnt) and dentry.d_name.name.is_valid():

        dname = dentry.d_name.name.dereference_as("String", length = MAX_STRING_LENGTH)

        ret_path.append(dname.strip('/'))

        if dentry == vfsmnt.mnt_root or dentry == dentry.d_parent:
            if vfsmnt.mnt_parent == vfsmnt.v():
                break
            dentry = vfsmnt.mnt_mountpoint
            vfsmnt = vfsmnt.mnt_parent
            continue

        parent = dentry.d_parent
        dentry = parent

    ret_path.reverse()

    if ret_path == []:
        return []

    ret_val = '/'.join([str(p) for p in ret_path if p != ""])

    if ret_val.startswith(("socket:", "pipe:")):
        if ret_val.find("]") == -1:
            ret_val = ret_val[:-1] + ":[{0}]".format(inode.i_ino)
        else:
            ret_val = ret_val.replace("/", "")

    elif ret_val != "inotify":
        ret_val = '/' + ret_val

    return ret_val

def get_path(task, filp):
    rdentry = task.fs.get_root_dentry()
    rmnt = task.fs.get_root_mnt()
    dentry = filp.dentry
    vfsmnt = filp.vfsmnt

    return do_get_path(rdentry, rmnt, dentry, vfsmnt)




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.utils as utils
import volatility.scan as scan
import volatility.plugins.common as common
import volatility.plugins.gui.windowstations as windowstations

class PoolScanAtom(scan.PoolScanner):
    """Pool scanner for atom tables"""

    def object_offset(self, found, address_space):
        """ This returns the offset of the object contained within
        this pool allocation.
        """
        pool_base = found - \
                self.buffer.profile.get_obj_offset('_POOL_HEADER', 'PoolTag')

        ## Note: all OS after XP, there are an extra 8 bytes (for 32-bit)
        ## or 16 bytes (for 64-bit) between the _POOL_HEADER and _RTL_ATOM_TABLE. 
        ## This is variable length structure, so we can't use the bottom-up
        ## approach as we do with other object scanners - because the size of an
        ## _RTL_ATOM_TABLE differs depending on the number of hash buckets. 

        build = (self.buffer.profile.metadata.get('major', 0),
                 self.buffer.profile.metadata.get('minor', 0))

        if self.buffer.profile.metadata.get('memory_model', '32bit') == '32bit':
            fixup = 8 if build > (5, 1) else 0
        else:
            fixup = 16 if build > (5, 1) else 0

        return pool_base + self.buffer.profile.get_obj_size('_POOL_HEADER') + fixup

    checks = [ ('PoolTagCheck', dict(tag = "AtmT")),
               ('CheckPoolSize', dict(condition = lambda x: x >= 200)),
               ('CheckPoolType', dict(paged = True, non_paged = True, free = True)),
               ]

class AtomScan(common.AbstractWindowsCommand):
    """Pool scanner for _RTL_ATOM_TABLE"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)

        config.add_option("SORT-BY", short_option = 'S', type = "choice",
                          choices = ["atom", "refcount", "offset"], default = "offset",
                          help = "Sort by [offset | atom | refcount]", action = "store")

    def calculate(self):
        flat_space = utils.load_as(self._config, astype = 'physical')
        kernel_space = utils.load_as(self._config)

        # Scan for atom tables
        for offset in PoolScanAtom().scan(flat_space):

            # There's no way to tell which session or window station 
            # owns an atom table by *just* looking at the atom table, 
            # so we have to instantiate it from the default kernel AS. 
            atom_table = obj.Object('_RTL_ATOM_TABLE', offset = offset,
                    vm = flat_space, native_vm = kernel_space)

            if atom_table.is_valid():
                yield atom_table

    def render_text(self, outfd, data):

        self.table_header(outfd,
                         [("TableOfs(P)", "[addr]"),
                          ("AtomOfs(V)", "[addrpad]"),
                          ("Atom", "[addr]"),
                          ("Refs", "6"),
                          ("Pinned", "6"),
                          ("Name", ""),
                         ])

        for atom_table in data:

            # This defeats the purpose of having a generator, but
            # its required if we want to be able to sort. We also
            # filter string atoms here. 
            atoms = [a for a in atom_table.atoms() if a.is_string_atom()]

            if self._config.SORT_BY == "atom":
                attr = "Atom"
            elif self._config.SORT_BY == "refcount":
                attr = "ReferenceCount"
            else:
                attr = "obj_offset"

            for atom in sorted(atoms, key = lambda x: getattr(x, attr)):

                self.table_row(outfd,
                    atom_table.obj_offset,
                    atom.obj_offset,
                    atom.Atom, atom.ReferenceCount,
                    atom.Pinned,
                    str(atom.Name or "")
                    )

class Atoms(common.AbstractWindowsCommand):
    """Print session and window station atom tables"""

    def calculate(self):
        seen = []

        # Find the atom tables that belong to each window station 
        for wndsta in windowstations.WndScan(self._config).calculate():

            offset = wndsta.obj_native_vm.vtop(wndsta.pGlobalAtomTable)
            if offset in seen:
                continue
            seen.append(offset)

            # The atom table is dereferenced in the proper 
            # session space 
            atom_table = wndsta.AtomTable

            if atom_table.is_valid():
                yield atom_table, wndsta

        # Find atom tables not linked to specific window stations. 
        # This finds win32k!UserAtomHandleTable. 
        for table in AtomScan(self._config).calculate():
            if table.PhysicalAddress not in seen:
                yield table, obj.NoneObject("No windowstation")

    def render_text(self, outfd, data):

        self.table_header(outfd,
                         [("Offset(P)", "[addr]"),
                          ("Session", "^10"),
                          ("WindowStation", "^18"),
                          ("Atom", "[addr]"),
                          ("RefCount", "^10"),
                          ("HIndex", "^10"),
                          ("Pinned", "^10"),
                          ("Name", ""),
                         ])

        for atom_table, window_station in data:
            for atom in atom_table.atoms():
            
                ## Filter string atoms 
                if not atom.is_string_atom():
                    continue 
            
                self.table_row(outfd,
                    atom_table.PhysicalAddress,
                    window_station.dwSessionId,
                    window_station.Name,
                    atom.Atom,
                    atom.ReferenceCount,
                    atom.HandleIndex,
                    atom.Pinned,
                    str(atom.Name or "")
                    )



# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.conf as conf
import urllib
import sys
import os
## This is required to ensure that LOCATION is defined here
import volatility.debug as debug
import volatility.addrspace as addrspace #pylint: disable-msg=W0611

config = conf.ConfObject()

def set_location(_option, _opt_str, value, parser):
    """Sets the location variable in the parser to the filename in question"""
    if not os.path.exists(os.path.abspath(value)):
        debug.error("The requested file doesn't exist")
    if parser.values.location == None:
        slashes = "//"
        # Windows pathname2url decides to convert C:\blah to ///C:/blah
        # So to keep the URLs correct, we only add file: rather than file://
        if sys.platform.startswith('win'):
            slashes = ""
        parser.values.location = "file:" + slashes + urllib.pathname2url(os.path.abspath(value))

config.add_option("FILENAME", default = None, action = "callback",
                  callback = set_location, type = 'str',
                  short_option = 'f', nargs = 1,
                  help = "Filename to use when opening an image")



# Volatility
#
# Copyright (c) 2008-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#
# Code found in WindowsHiberFileSpace32 for parsing meta information
# is inspired by the work of Matthieu Suiche:  http://sandman.msuiche.net/.
# A special thanks to Matthieu for all his help integrating 
# this code in Volatility.

""" A Hiber file Address Space """
import volatility.addrspace as addrspace
import volatility.obj as obj
import volatility.win32.xpress as xpress
import struct


#pylint: disable-msg=C0111

PAGE_SIZE = 0x1000
page_shift = 12

class Store(object):
    def __init__(self, limit = 50):
        self.limit = limit
        self.cache = {}
        self.seq = []
        self.size = 0

    def put(self, key, item):
        self.cache[key] = item
        self.size += len(item)

        self.seq.append(key)
        if len(self.seq) >= self.limit:
            key = self.seq.pop(0)
            self.size -= len(self.cache[key])
            del self.cache[key]

    def get(self, key):
        return self.cache[key]

class WindowsHiberFileSpace32(addrspace.BaseAddressSpace):
    """ This is a hibernate address space for windows hibernation files.

    In order for us to work we need to:
    1) have a valid baseAddressSpace
    2) the first 4 bytes must be 'hibr' or 'wake' 
        otherwise we bruteforce to find self.header.FirstTablePage in 
        _get_first_table_page() this occurs with a zeroed PO_MEMORY_IMAGE header
    """
    order = 10
    def __init__(self, base, config, **kwargs):
        self.as_assert(base, "No base Address Space")
        addrspace.BaseAddressSpace.__init__(self, base, config, **kwargs)
        self.runs = []
        self.PageDict = {}
        self.HighestPage = 0
        self.PageIndex = 0
        self.AddressList = []
        self.LookupCache = {}
        self.PageCache = Store(50)
        self.MemRangeCnt = 0
        self.entry_count = 0xFF

        # Extract header information
        self.as_assert(self.profile.has_type("PO_MEMORY_IMAGE"), "PO_MEMORY_IMAGE is not available in profile")
        self.header = obj.Object('PO_MEMORY_IMAGE', 0, base)

        ## Is the signature right?
        if self.header.Signature.lower() not in ['hibr', 'wake']:
            self.header = obj.NoneObject("Invalid hibernation header")

        volmag = obj.VolMagic(base)
        self.entry_count = volmag.HibrEntryCount.v()

        PROC_PAGE = volmag.HibrProcPage.v()

        # Check it's definitely a hibernation file
        self.as_assert(self._get_first_table_page() is not None, "No xpress signature found")

        # Extract processor state
        self.ProcState = obj.Object("_KPROCESSOR_STATE", PROC_PAGE * 4096, base)

        ## This is a pointer to the page table - any ASs above us dont
        ## need to search for it.
        self.dtb = self.ProcState.SpecialRegisters.Cr3.v()

        # This is a lengthy process, it was cached, but it may be best to delay this
        # until it's absolutely necessary and/or convert it into a generator...
        self.build_page_cache()

    def _get_first_table_page(self):
        if self.header != None:
            return self.header.FirstTablePage
        for i in range(10):
            if self.base.read(i * PAGE_SIZE, 8) == "\x81\x81xpress":
                return i - 1
        return None

    def build_page_cache(self):
        XpressIndex = 0
        XpressHeader = obj.Object("_IMAGE_XPRESS_HEADER",
                                  (self._get_first_table_page() + 1) * 4096,
                                  self.base)

        XpressBlockSize = self.get_xpress_block_size(XpressHeader)

        MemoryArrayOffset = self._get_first_table_page() * 4096

        while MemoryArrayOffset:
            MemoryArray = obj.Object('_PO_MEMORY_RANGE_ARRAY', MemoryArrayOffset, self.base)

            EntryCount = MemoryArray.MemArrayLink.EntryCount.v()
            for i in MemoryArray.RangeTable:
                start = i.StartPage.v()
                end = i.EndPage.v()
                LocalPageCnt = end - start
                self.as_assert((LocalPageCnt > 0), "Negative Page Count Range")

                if end > self.HighestPage:
                    self.HighestPage = end

                self.AddressList.append((start * 0x1000, LocalPageCnt * 0x1000))

                for j in range(0, LocalPageCnt):
                    if (XpressIndex and ((XpressIndex % 0x10) == 0)):
                        XpressHeader, XpressBlockSize = \
                                      self.next_xpress(XpressHeader, XpressBlockSize)

                    PageNumber = start + j
                    XpressPage = XpressIndex % 0x10
                    if XpressHeader.obj_offset not in self.PageDict:
                        self.PageDict[XpressHeader.obj_offset] = [
                            (PageNumber, XpressBlockSize, XpressPage)]
                    else:
                        self.PageDict[XpressHeader.obj_offset].append(
                            (PageNumber, XpressBlockSize, XpressPage))

                    ## Update the lookup cache
                    self.LookupCache[PageNumber] = (
                        XpressHeader.obj_offset, XpressBlockSize, XpressPage)

                    self.PageIndex += 1
                    XpressIndex += 1

            NextTable = MemoryArray.MemArrayLink.NextTable.v()

            # This entry count (EntryCount) should probably be calculated
            if (NextTable and (EntryCount == self.entry_count)):
                MemoryArrayOffset = NextTable * 0x1000
                self.MemRangeCnt += 1
                XpressHeader, XpressBlockSize = \
                                             self.next_xpress(XpressHeader, XpressBlockSize)

                # Make sure the xpress block is after the Memory Table
                while (XpressHeader.obj_offset < MemoryArrayOffset):
                    XpressHeader, XpressBlockSize = \
                        self.next_xpress(XpressHeader, 0)

                XpressIndex = 0
            else:
                MemoryArrayOffset = 0

    def next_xpress(self, XpressHeader, XpressBlockSize):
        XpressHeaderOffset = XpressBlockSize + XpressHeader.obj_offset + \
                             XpressHeader.size()

        ## We only search this far
        BLOCKSIZE = 1024
        original_offset = XpressHeaderOffset
        while 1:
            data = self.base.read(XpressHeaderOffset, BLOCKSIZE)
            Magic_offset = data.find("\x81\x81xpress")
            if Magic_offset >= 0:
                XpressHeaderOffset += Magic_offset
                break
            else:
                XpressHeaderOffset += len(data)

            ## Only search this far in advance
            if XpressHeaderOffset - original_offset > 10240:
                return None, None

        XpressHeader = obj.Object("_IMAGE_XPRESS_HEADER", XpressHeaderOffset, self.base)
        XpressBlockSize = self.get_xpress_block_size(XpressHeader)

        return XpressHeader, XpressBlockSize

    def get_xpress_block_size(self, xpress_header):
        u0B = xpress_header.u0B.v() << 24
        u0A = xpress_header.u0A.v() << 16
        u09 = xpress_header.u09.v() << 8

        Size = u0B + u0A + u09
        Size = Size >> 10
        Size = Size + 1

        if ((Size % 8) == 0):
            return Size
        return (Size & ~7) + 8

    def get_header(self):
        return self.header

    def get_base(self):
        return self.base

    def is_paging(self):
        return (self.ProcState.SpecialRegisters.Cr0.v() >> 31) & 1

    def is_pse(self):
        return (self.ProcState.SpecialRegisters.Cr4.v() >> 4) & 1

    def is_pae(self):
        return (self.ProcState.SpecialRegisters.Cr4.v() >> 5) & 1

    def get_addr(self, addr):
        page = addr >> page_shift
        if page in self.LookupCache:
            (hoffset, size, pageoffset) = self.LookupCache[page]
            return hoffset, size, pageoffset
        return None, None, None

    def get_block_offset(self, _xb, addr):
        page = addr >> page_shift
        if page in self.LookupCache:
            (_hoffset, _size, pageoffset) = self.LookupCache[page]
            return pageoffset
        return None

    def is_valid_address(self, addr):
        XpressHeaderOffset, _XpressBlockSize, _XpressPage = self.get_addr(addr)
        return XpressHeaderOffset != None

    def read_xpress(self, baddr, BlockSize):
        try:
            return self.PageCache.get(baddr)
        except KeyError:
            data_read = self.base.read(baddr, BlockSize)
            if BlockSize == 0x10000:
                data_uz = data_read
            else:
                data_uz = xpress.xpress_decode(data_read)

                self.PageCache.put(baddr, data_uz)

            return data_uz

    def _partial_read(self, addr, len):
        """ A function which reads as much as possible from the current page.

        May return a short read.
        """
        ## The offset within the page where we start
        page_offset = (addr & 0x00000FFF)

        ## How much data can we satisfy?
        available = min(PAGE_SIZE - page_offset, len)

        ImageXpressHeader, BlockSize, XpressPage = self.get_addr(addr)
        if not ImageXpressHeader:
            return None

        baddr = ImageXpressHeader + 0x20

        data = self.read_xpress(baddr, BlockSize)

        ## Each block decompressed contains 2**page_shift pages. We
        ## need to know which page to use here.
        offset = XpressPage * 0x1000 + page_offset

        return data[offset:offset + available]

    def read(self, addr, length, zread = False):
        result = ''
        while length > 0:
            data = self._partial_read(addr, length)
            if not data:
                break

            addr += len(data)
            length -= len(data)
            result += data

        if result == '':
            if zread:
                return ('\0' * length)
            result = obj.NoneObject("Unable to read data at " + str(addr) + " for length " + str(length))

        return result

    def zread(self, addr, length):
        stuff_read = self.read(addr, length, zread = True)
        return stuff_read

    def read_long(self, addr):
        _baseaddr = self.get_addr(addr)
        string = self.read(addr, 4)
        if not string:
            return obj.NoneObject("Could not read long at " + str(addr))
        (longval,) = struct.unpack('=I', string)
        return longval

    def get_available_pages(self):
        page_list = []
        for _i, xb in enumerate(self.PageDict.keys()):
            for page, _size, _offset in self.PageDict[xb]:
                page_list.append([page * 0x1000, 0x1000])
        return page_list

    def get_address_range(self):
        """ This relates to the logical address range that is indexable """
        size = self.HighestPage * 0x1000 + 0x1000
        return [0, size]

    def check_address_range(self, addr):
        memrange = self.get_address_range()
        if addr < memrange[0] or addr > memrange[1]:
            raise IOError

    def get_available_addresses(self):
        """ This returns the ranges  of valid addresses """
        for i in self.AddressList:
            yield i

    def close(self):
        self.base.close()




# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import sys
import timeit
import binascii
import xml.etree.cElementTree as etree
import volatility.commands as commands
import volatility.debug as debug
import volatility.utils as utils
PAGESIZE = 4096

#XML Example file format
#
#<patchfile>
#  <patchinfo method="pagescan">
#    <constraints>
#      <match offset="[offset within page]">DEADBEEFC0FFEE</match>
#      ...
#    </constraints>
#    <patches>
#      <patch offset="[offset within page]">BEEFF00DEE</match>
#    </patches>
#    ...
#  </patchinfo>
#</patchfile>

class MultiPageScanner(object):
    """Scans a page at a time through the address space
    
       Designed to minimize reads/writes to the address space
    """
    def __init__(self, patchers, full = False):
        self.patchers = list(patchers)
        self.maxlen = 0
        self.remove_patchers = not full

    def use_fullpage(self, address_space):
        """Calibrate the scanner to ensure fastest speed"""
        # Define the calibration functions
        timeit_fullpage = lambda: list(self.scan_page(address_space, 0, True))
        timeit_nonfullpage = lambda: list(self.scan_page(address_space, 0, False))

        with_fullpage = timeit.repeat(timeit_fullpage, number = 100)
        without_fullpage = timeit.repeat(timeit_nonfullpage, number = 100)
        return min(with_fullpage) < min(without_fullpage)

    def scan(self, address_space, outfd):
        """Scans through the pages"""
        page_offset = 0

        sys.stdout.write("Calibrating for speed: ")
        sys.stdout.flush()
        fullpage = self.use_fullpage(address_space)
        if fullpage:
            sys.stdout.write("Reading full pages\n")
        else:
            sys.stdout.write("Reading patch locations per page\n")
        sys.stdout.flush()

        done = False
        while address_space.is_valid_address(page_offset + PAGESIZE) and not done:
            sys.stdout.write("\rScanning: {0:08X}".format(page_offset))
            sys.stdout.flush()

            # Run through any patchers that didn't fail
            for patcher in self.scan_page(address_space, page_offset, fullpage):
                outfd.write("\rPatching {0} at page {1:x}\n".format(patcher.get_name(), page_offset))
                patcher.patch(address_space, page_offset)
                if self.remove_patchers:
                    self.patchers.remove(patcher)
                    # Stop if we've got nothing left to look for
                    if not len(self.patchers):
                        done = True

            # Jump to the next page
            page_offset += PAGESIZE
        sys.stdout.write("\n")

    def scan_page(self, address_space, page_offset, fullpage = False):
        """Runs through patchers for a single page"""
        if fullpage:
            pagedata = address_space.read(page_offset, PAGESIZE)

        for patcher in self.patchers:
            for offset, data in patcher.get_constraints():
                if fullpage:
                    testdata = pagedata[offset:offset + len(data)]
                else:
                    testdata = address_space.read(page_offset + offset, len(data))
                if data != testdata:
                    break
            else:
                yield patcher

class PatcherObject(object):
    """Simple object to hold patching data"""
    def __init__(self, name):
        self.name = name
        self.patches = set()
        self.constraints = set()

    def add_constraint(self, offset, data):
        """Adds a constraint to the constraintlist"""
        # Ensure that all offsets are within PAGESIZE
        self.constraints.add((offset % PAGESIZE, data))

    def add_patch(self, offset, patch):
        """Adds a patch to the patchlist"""
        # Ensure that all offsets are within PAGESIZE
        self.patches.add((offset % PAGESIZE, patch))

    def patch(self, addr_space, page_offset):
        """Writes to the address space"""
        result = True
        for offset, patch, in self.patches:
            result = result and addr_space.write(page_offset + offset, patch)
        return result

    def get_patches(self):
        """Returns the list of patches for this patcher"""
        return self.patches

    def get_constraints(self):
        return self.constraints

    def get_name(self):
        """Returns the name of the patcher"""
        return self.name

class Patcher(commands.Command):
    """Patches memory based on page scans"""
    def __init__(self, config, *args, **kwargs):
        commands.Command.__init__(self, config, *args, **kwargs)
        config.add_option('XML-INPUT', short_option = 'x',
                  help = 'Input XML file for patching binaries')

    def calculate(self):
        """Calculates the patchers"""
        addr_space = utils.load_as(self._config, astype = 'physical')
        scanner = MultiPageScanner(self.parse_patchfile())
        return scanner, addr_space

    def render_text(self, outfd, data):
        """Renders the text and carries out the patching"""
        scanner, addr_space = data
        scanner.scan(addr_space, outfd)

    def get_offset(self, tag):
        """Returns the offset from a tag"""
        offset = tag.get('offset', None)
        if not offset:
            return None
        base = 10
        if offset.startswith('0x'):
            offset = offset[2:]
            base = 16
        return int(offset, base)

    def parse_patchfile(self):
        """Parses the patch XML data"""
        if not self._config.WRITE:
            print "Warning: WRITE support not enabled, no patching will occur"

        if self._config.XML_INPUT is None:
            debug.error("No XML input file was specified")
        try:
            root = etree.parse(self._config.XML_INPUT).getroot()
        except SyntaxError, e:
            debug.error("XML input file was improperly formed: " + str(e))

        for element in root:
            if element.tag == 'patchinfo':
                if element.get('method', 'nomethod') == 'pagescan':
                    patcher = PatcherObject(element.get('name', 'Unlabelled'))
                    constraints = None
                    for tag in element:
                        if tag.tag == 'constraints':
                            constraints = tag
                        if tag.tag == 'patches':
                            patches = tag
                    if constraints is None:
                        debug.error("Patch input file does not contain any valid constraints")

                    # Parse the patches section
                    for tag in patches:
                        if tag.tag == 'setbytes':
                            offset = self.get_offset(tag)
                            data = binascii.a2b_hex(tag.text)
                            if offset is not None and len(data):
                                patcher.add_patch(offset, data)
                    if not len(patcher.get_patches()):
                        # No patches, no point adding this
                        break

                    # Parse the constraints section
                    for c in constraints:
                        if c.tag == 'match':
                            offset = self.get_offset(c)
                            data = binascii.a2b_hex(c.text)
                            if offset is not None and len(data):
                                patcher.add_constraint(offset, data)
                    yield patcher
                else:
                    debug.error("Unsupported patchinfo method " + element.method)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (c) 2012 Michael Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.taskmods as taskmods

class Envars(taskmods.DllList):
    "Display process environment variables"

    def render_text(self, outfd, data):

        self.table_header(outfd,
            [("Pid", "8"),
             ("Process", "20"),
             ("Block", "[addrpad]"),
             ("Variable", "30"),
             ("Value", ""),
            ])

        for task in data:
            for var, val in task.environment_variables():
                self.table_row(outfd,
                    task.UniqueProcessId,
                    task.ImageFileName,
                    task.Peb.ProcessParameters.Environment, 
                    var, val
                    )



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import struct
from operator import attrgetter
import volatility.obj as obj
import volatility.debug as debug
import volatility.addrspace as addrspace
import volatility.plugins.linux.common  as linux_common
import volatility.plugins.linux.pslist as linux_pslist

bash_vtypes_32 = {
    '_hist_entry': [ 0xc, {
    'line': [0x0, ['pointer', ['String', dict(length = 1024)]]],
    'timestamp': [0x4, ['pointer', ['String', dict(length = 1024)]]],
    'data': [0x8, ['pointer', ['void']]],
    }],
}

bash_vtypes_64 = {
    '_hist_entry': [ 24, {
    'line': [0, ['pointer', ['String', dict(length = 1024)]]],
    'timestamp': [8, ['pointer', ['String', dict(length = 1024)]]],
    'data': [16, ['pointer', ['void']]],
    }],
}

class _hist_entry(obj.CType):
    """A class for history entries"""

    def is_valid(self):
        
        # Check the basic structure members 
        if (not obj.CType.is_valid(self) or  
                    not self.line.is_valid() or 
                    len(self.line.dereference()) == 0 or  
                    not self.timestamp.is_valid()):
            return False

        # A pointer to the timestamp string 
        ts = self.timestamp.dereference()

        # At this point in time, the epoc integer size will 
        # never be less than 10 characters, and the stamp is 
        # always preceded by a pound/hash character. 
        if len(ts) < 10 or str(ts)[0] != "#":
            return False

        # The final check is to make sure the entire string
        # is composed of numbers. Try to convert to an int. 
        try:
            int(str(ts)[1:])
        except ValueError:
            return False 

        return True

    @property
    def time_as_integer(self):
        # Get the string and remove the leading "#" from the timestamp 
        time_string = str(self.timestamp.dereference())[1:] 
        # Convert the string into an integer (number of seconds)
        return int(time_string)

    def time_object(self):
        nsecs = self.time_as_integer
        # Build a timestamp object from the integer 
        time_val = struct.pack("<I", nsecs)
        time_buf = addrspace.BufferAddressSpace(self.obj_vm.get_config(), data = time_val)
        time_obj = obj.Object("UnixTimeStamp", offset = 0, vm = time_buf, is_utc = True)
        return time_obj

class BashTypes(obj.ProfileModification):
    conditions = {"os" : lambda x : x in ["linux", "mac"]}

    def modification(self, profile):
        
        if profile.metadata.get('memory_model', '32bit') == "32bit":
            profile.vtypes.update(bash_vtypes_32)
        else:
            profile.vtypes.update(bash_vtypes_64)

        profile.object_classes.update({"_hist_entry": _hist_entry})

class linux_bash(linux_pslist.linux_pslist):
    """Recover bash history from bash process memory"""

    def __init__(self, config, *args, **kwargs): 
        linux_pslist.linux_pslist.__init__(self, config, *args, **kwargs)
        self._config.add_option('PRINTUNALLOC', short_option = 'P', default = None, help = 'print unallocated entries, please redirect to a file', action = 'store_true')
        self._config.add_option('HISTORY_LIST', short_option = 'H', default = None, help = 'address from history_list - see the Volatility wiki', action = 'store', type = 'long')        
        self._config.add_option('SCAN_ALL', short_option = 'A', default = False, help = 'scan all processes, not just those named bash', action = 'store_true')    

    def calculate(self):
        linux_common.set_plugin_members(self)
    
        tasks = linux_pslist.linux_pslist(self._config).calculate()

        for task in tasks:
            proc_as = task.get_process_address_space()
            
            # In cases when mm is an invalid pointer 
            if not proc_as:
                continue

            if not self._config.HISTORY_LIST:
                # Do we scan everything or just /bin/bash instances?
                if not (self._config.SCAN_ALL or str(task.comm) == "bash"):
                    continue

                # Keep a bucket of history objects so we can order them
                history_entries = []

                # Brute force the history list of an address isn't provided 
                ts_offset = proc_as.profile.get_obj_offset("_hist_entry", "timestamp") 

                # Are we dealing with 32 or 64-bit pointers
                if proc_as.profile.metadata.get('memory_model', '32bit') == '32bit':
                    pack_format = "I"
                else:
                    pack_format = "Q"

                # Look for strings that begin with pound/hash on the process heap 
                for ptr_hash in task.search_process_memory(["#"], heap_only = True):
                    
                    # Find pointers to this strings address, also on the heap 
                    addr = struct.pack(pack_format, ptr_hash)

                    for ptr_string in task.search_process_memory([addr], heap_only = True):
                        
                        # Check if we found a valid history entry object 
                        hist = obj.Object("_hist_entry", 
                                          offset = ptr_string - ts_offset, 
                                          vm = proc_as)

                        if hist.is_valid():
                            history_entries.append(hist)
                            # We can terminate this inner loop now 
                            break
                
                # Report everything we found in order
                for hist in sorted(history_entries, key = attrgetter('time_as_integer')):
                    yield task, hist              
            else:    
                the_history_addr = the_history_addr = self._config.HISTORY_LIST
                the_history = obj.Object("Pointer", vm = proc_as, offset = the_history_addr)
                max_ents = 2001
                the_history = obj.Object(theType = 'Array', offset = the_history, 
                                         vm = proc_as, targetType = 'Pointer', 
                                         count = max_ents)

                for ptr in the_history:
                    if not ptr:
                        if self._config.PRINTUNALLOC:
                            continue
                        else:
                            break

                    hist = ptr.dereference_as("_hist_entry")      
    
                    if hist.is_valid():
                        yield task, hist
    
    def render_text(self, outfd, data):

        self.table_header(outfd, [("Pid", "8"), 
                                  ("Name", "20"),
                                  ("Command Time", "30"),
                                  ("Command", ""),])
                                    
        for task, hist_entry in data:
            self.table_row(outfd, task.pid, task.comm, 
                           hist_entry.time_object(), 
                           hist_entry.line.dereference())
            





# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.plugins.mac.lsmod as lsmod

class mac_notifiers(lsmod.mac_lsmod):
    """ Detects rootkits that add hooks into I/O Kit (e.g. LogKext) """

    def _struct_or_class(self, type_name):
        """Return the name of a structure or class. 

        More recent versions of OSX define some types as 
        classes instead of structures, so the naming is
        a little different.   
        """
        if self.addr_space.profile.vtypes.has_key(type_name):
            return type_name
        else:
            return type_name + "_class"

    def calculate(self):
        common.set_plugin_members(self)

        (kernel_symbol_addresses, kmods) = common.get_kernel_addrs(self)
        gnotify_addr = common.get_cpp_sym("gNotifications", self.addr_space.profile)
        p = obj.Object("Pointer", offset = gnotify_addr, vm = self.addr_space)
        gnotifications = p.dereference_as(self._struct_or_class("OSDictionary"))

        ents = obj.Object('Array', offset = gnotifications.dictionary, 
                          vm = self.addr_space, 
                          targetType = self._struct_or_class("dictEntry"), 
                          count = gnotifications.count)

        # walk the current set of notifications
        for ent in ents:

            if ent == None:
                continue

            key = str(ent.key.dereference_as(self._struct_or_class("OSString")))

            # get the value
            valset = ent.value.dereference_as(self._struct_or_class("OSOrderedSet"))

            notifiers_ptrs = obj.Object('Array', offset = valset.array, 
                                        vm = self.addr_space, 
                                        targetType = 'Pointer', 
                                        count = valset.count)
            
            for ptr in notifiers_ptrs:
                notifier = ptr.dereference_as(self._struct_or_class("_IOServiceNotifier"))

                if notifier == None:
                    continue

                matches = self.get_matching(notifier)

                # this is the function that handles whatever the notification is for
                # this should be only in the kernel or in one of the known IOKit 
                # drivers for the specific kernel
                handler = notifier.handler

                good = common.is_known_address(handler, kernel_symbol_addresses, kmods)
                yield (good, key, notifier, matches)

    # returns the list of matching notifiers (serviceMatch) for a notifier as a string
    def get_matching(self, notifier):
        matches = []
    
        ents = obj.Object('Array', offset = notifier.matching.dictionary, 
                          vm = self.addr_space, 
                          targetType = self._struct_or_class("dictEntry"), 
                          count = notifier.matching.count)

        for ent in ents:
            if ent == None:
                continue

            match = ent.value.dereference_as(self._struct_or_class("OSString"))        
            matches.append(str(match))

        return ",".join(matches)

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Status", "10"), 
                                  ("Key", "30"), 
                                  ("Handler", "[addrpad]"), 
                                  ("Matches", "")])

        for (good, key, notifier, matches) in data:

            if good == 0:
                status = "UNKNOWN"
            else:
                status = "OK"

            self.table_row(outfd, status, key, notifier.handler, matches)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization:
"""

import os
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common
from volatility.plugins.linux.slab_info import linux_slabinfo

class linux_sk_buff_cache(linux_common.AbstractLinuxCommand):
    """Recovers packets from the sk_buff kmem_cache"""

    def __init__(self, config, *args, **kwargs):
        self.edir = None
        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)
        self._config.add_option('UNALLOCATED', short_option = 'u', default = False, help = 'Show unallocated', action = 'store_true')
        self._config.add_option('DUMP-DIR', short_option = 'D', default = None, help = 'output directory for recovered packets', action = 'store', type = 'str')

    def write_sk_buff(self, s):
        pkt_len = s.len

        # keep sane sized packets
        if 0 < pkt_len < 0x6400000:

            start = s.data

            data = self.addr_space.zread(start, pkt_len)

            fname = "{0:x}".format(s.obj_offset)
            fd = open(os.path.join(self.edir, fname), "wb")
            fd.write(data)
            fd.close()

            yield "Wrote {0:d} bytes to {1:s}".format(pkt_len, fname)

    def walk_cache(self, cache_name):
        cache = linux_slabinfo(self._config).get_kmem_cache(cache_name, self._config.UNALLOCATED, struct_name = "sk_buff")

        if not cache:
            return

        for s in cache:
            for msg in self.write_sk_buff(s):
                yield msg

    def calculate(self):
        linux_common.set_plugin_members(self)

        self.edir = self._config.DUMP_DIR

        if not self.edir:
            debug.error("No output directory given.")

        for msg in self.walk_cache("skbuff_head_cache"):
            yield msg

        for msg in self.walk_cache("skbuff_fclone_cache"):
            yield msg

    def render_text(self, outfd, data):

        for msg in data:
            outfd.write("{0:s}\n".format(msg))





# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import time
import urlparse
import volatility.addrspace as addrspace

# TODO: Remove this once we no longer support old/broken versions of urlparse (2.6.2)
check = urlparse.urlsplit("firewire://method/0")
urlparse_broken = False
if check[1] != 'method':
  urlparse_broken = True

def FirewireRW(netloc, location):
    if netloc in fw_implementations:
        return fw_implementations[netloc](location)
    return None

class FWRaw1394(object):
    def __init__(self, location):
        locarr = location.split('/')
        self.bus = locarr[0]
        self.node = locarr[1]
        self._node = None

    def is_valid(self):
        """Initializes the firewire implementation"""
        self._node = None
        try:
            h = firewire.Host()
            self._node = h[self.bus][self.node]
            return True, "Valid"
        except IndexError:
            return False, "Firewire node " + str(self.node) + " on bus " + str(self.bus) + " was not accessible"
        except IOError, e:
            return False, "Firewire device IO error - " + str(e)
        return False, "Unknown Error occurred"

    def read(self, addr, length):
        """Reads bytes from the specified address"""
        return self._node.read(addr, length)

    def write(self, addr, buf):
        """Writes buf bytes at addr"""
        return self._node.write(addr, buf)

class FWForensic1394(object):
    def __init__(self, location):
        """Initializes the firewire implementation"""
        self.location = location.strip('/')
        self._bus = forensic1394.Bus()
        self._bus.enable_sbp2()
        self._device = None

    def is_valid(self):
        try:
            time.sleep(2)
            devices = self._bus.devices()
            # FIXME: Base the device off the location rather than hardcoded first remote device
            self._device = devices[int(self.location)]
            # Cetain Firewire cards misreport their maximum request size, notably Ricoh onboard chipsets
            # Uncomment the line below for such broken hardware
            # self._device._request_size = 1024
            if not self._device.isopen():
                self._device.open()
            # The device requires time to settle before it can be used
            return True, "Valid"
        except IOError, e:
            print repr(e)
            return False, "Forensic1394 returned an exception: " + str(e)
        return False, "Unknown Error occurred"

    def read(self, addr, length):
        """Reads bytes from the specified address"""
        return self._device.read(addr, length)

    def write(self, addr, buf):
        """Writes buf bytes at addr"""
        return self._device.write(addr, buf)

class FirewireAddressSpace(addrspace.BaseAddressSpace):
    """A physical layer address space that provides access via firewire"""

    ## We should be *almost* the AS of last resort
    order = 99
    def __init__(self, base, config, **kargs):
        self.as_assert(base == None, 'Must be first Address Space')
        try:
            (scheme, netloc, path, _, _, _) = urlparse.urlparse(config.LOCATION)
            self.as_assert(scheme == 'firewire', 'Not a firewire URN')
            if urlparse_broken:
                if path.startswith('//') and path[2:].find('/') > 0:
                    firstslash = path[2:].find('/')
                    netloc = path[2:firstslash + 2]
                    path = path[firstslash + 3:]
            self._fwimpl = FirewireRW(netloc, path)
        except (AttributeError, ValueError):
            self.as_assert(False, "Unable to parse {0} as a URL".format(config.LOCATION))
        addrspace.BaseAddressSpace.__init__(self, base, config, **kargs)
        self.as_assert(self._fwimpl is not None, "Unable to locate {0} implementation.".format(netloc))
        valid, reason = self._fwimpl.is_valid()
        self.as_assert(valid, reason)

        # We have a list of exclusions because we know that trying to read anything in these sections
        # will cause the target machine to bluescreen
        # Exceptions are in the form (start, length, "Reason")
        self._exclusions = sorted([(0xa0000, 0xfffff - 0xa0000, "Upper Memory Area")])

        self.name = "Firewire using " + str(netloc) + " at " + str(path)
        # We have no way of knowing how big a firewire space is...
        # Set it to the maximum for the moment
        # TODO: Find a way of determining the size safely and reliably from the space itself 
        self.size = 0xFFFFFFFF

    def intervals(self, start, size):
        """Returns a list of intervals, from start of length size, that do not include the exclusions"""
        return self._intervals(sorted(self._exclusions), start, size + start, [])

    def _intervals(self, exclusions, start, end, accumulator):
        """Accepts a sorted list of intervals and a start and end
        
           This will return a list of intervals between start and end
           that does not contain any of the intervals in the list of exclusions.
        """
        if not len(exclusions):
            # We're done
            return accumulator + [(start, end - start)]

        e = exclusions[0]
        estart = e[0]
        eend = e[1] + estart

        # e and range overlap
        if (eend < start or estart > end):
            # Ignore this exclusion
            return self._intervals(exclusions[1:], start, end, accumulator)
        if estart < start:
            if eend < end:
                # Covers the start of the remaining length
                return self._intervals(exclusions[1:], eend, end, accumulator)
            else:
                # Covers the entire remaining area
                return accumulator
        else:
            if eend < end:
                # Covers a section of the remaining length
                return self._intervals(exclusions[1:], eend, end, accumulator + [(start, estart - start)])
            else:
                # Covers the end of the remaining length
                return accumulator + [(start, estart - start)]

    def read(self, offset, length):
        """Reads a specified size in bytes from the current offset
        
           Fills any excluded holes with zeros (so in that sense, similar to zread)
        """
        ints = self.intervals(offset, length)
        output = "\x00" * length
        try:
            for i in ints:
                datstart, datlen = i[0], i[1]
                if datlen > 0:
                    # node.read won't work on 0 byte
                    readdata = self._fwimpl.read(datstart, datlen)
                    # I'm not sure why, but sometimes readdata comes out longer than the requested size
                    # We just truncate it to the right length
                    output = output[:datstart - offset] + readdata[:datlen] + output[(datstart - offset) + datlen:]
        except IOError, e:
            print repr(e)
            raise RuntimeError("Failed to read from firewire device")
        self.as_assert(len(output) == length, "Firewire read lengths failed to match")
        return output

    def zread(self, offset, length):
        """ Delegate padded reads to normal read, since errors reading 
            the physical address should probably be reported back to the user
        """
        return self.read(offset, length)

    def write(self, offset, data):
        """Writes a specified size in bytes"""
        if not self._config.WRITE:
            return False

        ints = self.intervals(offset, len(data))
        try:
            for i in ints:
                datstart, datlen = i[0], i[1]
                if datlen > 0:
                    self._fwimpl.write(datstart, data[(datstart - offset):(datstart - offset) + datlen])
        except IOError:
            raise RuntimeError("Failed to write to the firewire device")
        return True

    def get_address_range(self):
        """Returns the size of the address range"""
        return [0, self.size - 1]

    def get_available_addresses(self):
        """Returns a list of available addresses"""
        for i in self.intervals(0, self.size):
            yield i

fw_implementations = {}

try:
    import firewire #pylint: disable-msg=F0401
    fw_implementations['raw1394'] = FWRaw1394
except ImportError:
    pass

try:
    import forensic1394 #pylint: disable-msg=F0401
    fw_implementations['forensic1394'] = FWForensic1394
except ImportError:
    pass

if not len(fw_implementations):
    FirewireAddressSpace = None



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Brendan Dolan-Gavitt
@license:      GNU General Public License 2.0
@contact:      bdolangavitt@wesleyan.edu
"""

#pylint: disable-msg=C0111

import volatility.win32.rawreg as rawreg
import volatility.win32.hive as hive
import volatility.win32.lsasecrets as lsasecrets
import volatility.win32.hashdump as hashdump
from Crypto.Hash import HMAC
from Crypto.Cipher import ARC4
from struct import unpack

def get_nlkm(secaddr, lsakey):
    return lsasecrets.get_secret_by_name(secaddr, 'NL$KM', lsakey)

def decrypt_hash(edata, nlkm, ch):
    hmac_md5 = HMAC.new(nlkm, ch)
    rc4key = hmac_md5.digest()

    rc4 = ARC4.new(rc4key)
    data = rc4.encrypt(edata)
    return data

def parse_cache_entry(cache_data):
    (uname_len, domain_len) = unpack("<HH", cache_data[:4])
    (domain_name_len,) = unpack("<H", cache_data[60:62])
    ch = cache_data[64:80]
    enc_data = cache_data[96:]
    return (uname_len, domain_len, domain_name_len, enc_data, ch)

def parse_decrypted_cache(dec_data, uname_len,
        domain_len, domain_name_len):
    uname_off = 72
    pad = 2 * ((uname_len / 2) % 2)
    domain_off = uname_off + uname_len + pad
    pad = 2 * ((domain_len / 2) % 2)
    domain_name_off = domain_off + domain_len + pad

    hashh = dec_data[:0x10]
    username = dec_data[uname_off:uname_off + uname_len]
    username = username.decode('utf-16-le')
    domain = dec_data[domain_off:domain_off + domain_len]
    domain = domain.decode('utf-16-le')
    domain_name = dec_data[domain_name_off:domain_name_off + domain_name_len]
    domain_name = domain_name.decode('utf-16-le')

    return (username, domain, domain_name, hashh)

def dump_hashes(sysaddr, secaddr):
    bootkey = hashdump.get_bootkey(sysaddr)
    if not bootkey:
        return None

    lsakey = lsasecrets.get_lsa_key(secaddr, bootkey)
    if not lsakey:
        return None

    nlkm = get_nlkm(secaddr, lsakey)
    if not nlkm:
        return None

    root = rawreg.get_root(secaddr)
    if not root:
        return None

    cache = rawreg.open_key(root, ["Cache"])
    if not cache:
        return None

    hashes = []
    for v in rawreg.values(cache):
        if v.Name == "NL$Control":
            continue

        data = v.obj_vm.read(v.Data, v.DataLength)

        (uname_len, domain_len, domain_name_len,
            enc_data, ch) = parse_cache_entry(data)

        # Skip if nothing in this cache entry
        if uname_len == 0:
            continue

        dec_data = decrypt_hash(enc_data, nlkm, ch)

        (username, domain, domain_name,
            hashh) = parse_decrypted_cache(dec_data, uname_len,
                    domain_len, domain_name_len)

        hashes.append((username, domain, domain_name, hashh))

    return hashes

def dump_memory_hashes(addr_space, config, syshive, sechive):
    sysaddr = hive.HiveAddressSpace(addr_space, config, syshive)
    secaddr = hive.HiveAddressSpace(addr_space, config, sechive)

    for (u, d, dn, hashh) in dump_hashes(sysaddr, secaddr):
        print "{0}:{1}:{2}:{3}".format(u.lower(), hashh.encode('hex'),
                                       d.lower(), dn.lower())

def dump_file_hashes(syshive_fname, sechive_fname):
    sysaddr = hive.HiveFileAddressSpace(syshive_fname)
    secaddr = hive.HiveFileAddressSpace(sechive_fname)

    for (u, d, dn, hashh) in dump_hashes(sysaddr, secaddr):
        print "{0}:{1}:{2}:{3}".format(u.lower(), hashh.encode('hex'),
                                       d.lower(), dn.lower())
